@@ -30,9 +30,12 @@
 #include "layout.h"
 #include "metadata.h"
 #include "menu.h"
+#include "misc.h"
 #include "rcfile.h"
 #include "thumb.h"
 #include "ui_menu.h"
+#include "uri_utils.h"
+#include "ui_utildlg.h"
 
 #include <clutter-gtk/clutter-gtk.h>
 #include <champlain/champlain.h>
@@ -43,6 +46,8 @@
 #define THUMB_COLOUR 0xff, 0xff, 0xff, 0xff
 #define THUMB_SIZE 100
 
+#define DIRECTION_SIZE 300
+
 /*
  *-------------------------------------------------------------------
  * GPS Map utils
@@ -70,8 +75,203 @@ struct _PaneGPSData
 	gint selection_count;
 	gboolean centre_map_checked;
 	gboolean enable_markers_checked;
+	gdouble dest_latitude;
+	gdouble dest_longitude;
+	GList *geocode_list;
 };
 
+/*
+ *-------------------------------------------------------------------
+ * drag-and-drop
+ *-------------------------------------------------------------------
+ */
+enum {
+	TARGET_APP_COLLECTION_MEMBER,
+	TARGET_APP_EXIF_ENTRY,
+	TARGET_APP_KEYWORD_PATH,
+	TARGET_URI_LIST,
+	TARGET_TEXT_PLAIN
+};
+
+static GtkTargetEntry bar_pane_gps_drop_types[] = {
+	{ "text/uri-list", 0, TARGET_URI_LIST },
+	{ "text/plain", 0, TARGET_TEXT_PLAIN }
+};
+static gint n_gps_entry_drop_types = 2;
+
+static void bar_pane_gps_close_cancel_cb(GenericDialog *gd, gpointer data)
+{
+	PaneGPSData *pgd = data;
+
+	g_list_free(pgd->geocode_list);
+}
+
+static void bar_pane_gps_close_save_cb(GenericDialog *gd, gpointer data)
+{
+	PaneGPSData *pgd = data;
+	FileData *fd;
+	GList *work;
+
+	work = g_list_first(pgd->geocode_list);
+	while (work)
+		{
+		fd = work->data;
+		if (fd->name && !fd->parent)
+			{
+			work = work->next;
+			metadata_write_GPS_coord(fd, "Xmp.exif.GPSLatitude", pgd->dest_latitude);
+			metadata_write_GPS_coord(fd, "Xmp.exif.GPSLongitude", pgd->dest_longitude);
+			}
+		}
+	g_list_free(work);
+	g_list_free(pgd->geocode_list);
+}
+
+ static void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *context,
+									  gint x, gint y,
+									  GtkSelectionData *selection_data, guint info,
+									  guint time, gpointer data)
+{
+	PaneGPSData *pgd;
+	GenericDialog *gd;
+	FileData *fd, *fd_found;
+	GList *work, *list;
+	gint count, geocoded_count;
+	gdouble latitude, longitude;
+	GString *message;
+	gchar *location;
+	gchar **latlong;
+
+	pgd = g_object_get_data(G_OBJECT(pane), "pane_data");
+	if (!pgd) return;
+
+	if (info == TARGET_URI_LIST)
+		{
+		pgd->dest_longitude = champlain_view_x_to_longitude(CHAMPLAIN_VIEW(pgd->gps_view), x);
+		pgd->dest_latitude = champlain_view_y_to_latitude(CHAMPLAIN_VIEW(pgd->gps_view), y);
+
+		count = 0;
+		geocoded_count = 0;
+		pgd->geocode_list = NULL;
+
+		list = uri_filelist_from_gtk_selection_data(selection_data);
+
+		if (list)
+			{
+			work = list;
+			while (work)
+				{
+				fd = work->data;
+				work = work->next;
+				if (fd->name && !fd->parent)
+					{
+					count++;
+					pgd->geocode_list = g_list_append(pgd->geocode_list, fd);
+					latitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLatitude", 1000);
+					longitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLongitude", 1000);
+					if (latitude != 1000 && longitude != 1000)
+						{
+						geocoded_count++;
+						}
+					}
+				}
+			g_list_free(work);
+
+			if(count)
+				{
+				message = g_string_new("");
+				if (count == 1)
+					{
+					fd_found = g_list_first(pgd->geocode_list)->data;
+					g_string_append_printf(message,
+							_("\nDo you want to geocode image %s?"), fd_found->name);
+					}
+				else
+					{
+					g_string_append_printf(message,
+							_("\nDo you want to geocode %i images?"), count);
+					}
+				if (geocoded_count == 1 && count == 1)
+					{
+					g_string_append_printf(message,
+							_("\nThis image is already geocoded!"));
+					}
+				else if (geocoded_count == 1 && count > 1)
+					{
+					g_string_append_printf(message,
+							_("\nOne image is already geocoded!"));
+					}
+				else if (geocoded_count > 1 && count > 1)
+					{
+					g_string_append_printf(message,
+							_("\n%i Images are already geocoded!"), geocoded_count);
+					}
+
+				location = g_strdup_printf("%lf %lf", pgd->dest_latitude,
+														pgd->dest_longitude);
+				g_string_append_printf(message, _("\n\nPosition: %s \n"), location);
+
+				gd = generic_dialog_new(_("Geocode images"),
+							"geocode_images", NULL, TRUE,
+							bar_pane_gps_close_cancel_cb, pgd);
+				generic_dialog_add_message(gd, GTK_STOCK_DIALOG_QUESTION,
+							_("Write lat/long to meta-data?"),
+							message->str, TRUE);
+
+				generic_dialog_add_button(gd, GTK_STOCK_SAVE, NULL,
+												bar_pane_gps_close_save_cb, TRUE);
+
+				gtk_widget_show(gd->dialog);
+				g_free(location);
+				g_string_free(message, TRUE);
+				}
+			}
+		}
+
+	if (info == TARGET_TEXT_PLAIN)
+		{
+		location = decode_geo_parameters((gchar *)gtk_selection_data_get_data(selection_data));
+		if (!(g_strstr_len(location,-1,"Error")))
+			{
+			latlong = g_strsplit(location, " ", 2);
+			champlain_view_center_on(CHAMPLAIN_VIEW(pgd->gps_view),
+							g_ascii_strtod(latlong[0],NULL),
+							g_ascii_strtod(latlong[1],NULL));
+			g_strfreev(latlong);
+			}
+		g_free(location);
+		}
+
+	return;
+}
+
+static void bar_pane_gps_dnd_init(gpointer data)
+{
+	PaneGPSData *pgd = data;
+
+	gtk_drag_dest_set(pgd->widget,
+			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
+			  bar_pane_gps_drop_types, n_gps_entry_drop_types,
+			  GDK_ACTION_COPY | GDK_ACTION_MOVE);
+	g_signal_connect(G_OBJECT(pgd->widget), "drag_data_received",
+			 G_CALLBACK(bar_pane_gps_dnd_receive), NULL);
+
+}
+
+static gboolean bar_gps_draw_direction (ClutterCanvas *canvas,
+				cairo_t *cr, gpointer data)
+{
+	cairo_set_source_rgb(cr, 255, 0, 0);
+
+	cairo_set_line_width(cr, 2);
+	cairo_move_to(cr, 0, 1);
+	cairo_line_to(cr, DIRECTION_SIZE, 1);
+
+	cairo_stroke(cr);
+
+	return TRUE;
+}
+
 static void bar_pane_gps_thumb_done_cb(ThumbLoader *tl, gpointer data)
 {
 	FileData *fd;
@@ -98,12 +298,12 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 {
 	//PaneGPSData *pgd = data;
 	FileData *fd;
-	ClutterActor *marker;
+	ClutterActor *label_marker, *parent_marker;
 	ClutterColor marker_colour = { MARKER_COLOUR };
 	ClutterColor text_colour = { TEXT_COLOUR };
 	ClutterColor thumb_colour = { THUMB_COLOUR };
 	gchar *current_text;
-	ClutterActor *actor;
+	ClutterActor *actor, *direction;
 	ClutterActor *current_image;
 	GString *text;
 	gint height, width, rotate;
@@ -112,19 +312,19 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 
 	if (bevent->button == MOUSE_BUTTON_LEFT)
 		{
-		marker = CLUTTER_ACTOR(widget);
-		fd = g_object_get_data(G_OBJECT(marker), "file_fd");
+		label_marker = CLUTTER_ACTOR(widget);
+		fd = g_object_get_data(G_OBJECT(label_marker), "file_fd");
 
 		/* If the marker is showing a thumbnail, delete it
 		 */
-		current_image = champlain_label_get_image(CHAMPLAIN_LABEL(marker));
+		current_image = champlain_label_get_image(CHAMPLAIN_LABEL(label_marker));
 		if (current_image != NULL)
 			{
 			clutter_actor_destroy(CLUTTER_ACTOR(current_image));
-		 	champlain_label_set_image(CHAMPLAIN_LABEL(marker), NULL);
+		 	champlain_label_set_image(CHAMPLAIN_LABEL(label_marker), NULL);
 			}
 
-		current_text = g_strdup(champlain_label_get_text(CHAMPLAIN_LABEL(marker)));
+		current_text = g_strdup(champlain_label_get_text(CHAMPLAIN_LABEL(label_marker)));
 
 		/* If the marker is showing only the text character, replace it with a
 		 * thumbnail and date and altitude
@@ -139,7 +339,7 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 				{
 				actor = gtk_clutter_texture_new();
 				gtk_clutter_texture_set_from_pixbuf(GTK_CLUTTER_TEXTURE(actor), fd->thumb_pixbuf, NULL);
-				champlain_label_set_image(CHAMPLAIN_LABEL(marker), actor);
+				champlain_label_set_image(CHAMPLAIN_LABEL(label_marker), actor);
 				}
 			else if (fd->pixbuf != NULL)
 				{
@@ -164,7 +364,7 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 					gtk_clutter_texture_set_from_pixbuf(GTK_CLUTTER_TEXTURE(actor),
 										gdk_pixbuf_rotate_simple(gdk_pixbuf_scale_simple(fd->pixbuf, THUMB_SIZE, height * THUMB_SIZE / width,
 										GDK_INTERP_NEAREST), rotate), NULL);
-					champlain_label_set_image(CHAMPLAIN_LABEL(marker), actor);
+					champlain_label_set_image(CHAMPLAIN_LABEL(label_marker), actor);
 				}
 			else
 				{
@@ -173,7 +373,7 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 											bar_pane_gps_thumb_done_cb,
 											bar_pane_gps_thumb_error_cb,
 											NULL,
-											marker);
+											label_marker);
 				thumb_loader_start(tl, fd);
 				}
 
@@ -187,22 +387,30 @@ static gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButton
 				g_string_append(text, altitude);
 				}
 
-			champlain_label_set_text(CHAMPLAIN_LABEL(marker), text->str);
-			champlain_label_set_color(CHAMPLAIN_LABEL(marker), &thumb_colour);
-			champlain_label_set_text_color(CHAMPLAIN_LABEL(marker), &text_colour);
-			champlain_label_set_font_name(CHAMPLAIN_LABEL(marker), "sans 8");
+			champlain_label_set_text(CHAMPLAIN_LABEL(label_marker), text->str);
+			champlain_label_set_font_name(CHAMPLAIN_LABEL(label_marker), "sans 8");
+			champlain_marker_set_selection_color(&thumb_colour);
+			champlain_marker_set_selection_text_color(&text_colour);
 
 			g_free(altitude);
 			g_string_free(text, TRUE);
+
+			parent_marker = clutter_actor_get_parent(label_marker);
+			direction = clutter_actor_get_child_at_index(parent_marker, 0);
+			clutter_actor_set_opacity(direction, 255);
 			}
 		/* otherwise, revert to the hidden text marker
 		 */
 		else
 			{
-			champlain_label_set_text(CHAMPLAIN_LABEL(marker), "i");
-			champlain_label_set_color(CHAMPLAIN_LABEL(marker), &marker_colour);
-			champlain_label_set_text_color(CHAMPLAIN_LABEL(marker), &marker_colour);
-			champlain_label_set_font_name(CHAMPLAIN_LABEL(marker), "courier 5");
+			champlain_label_set_text(CHAMPLAIN_LABEL(label_marker), "i");
+			champlain_label_set_font_name(CHAMPLAIN_LABEL(label_marker), "courier 5");
+			champlain_marker_set_selection_color(&marker_colour);
+			champlain_marker_set_selection_text_color(&marker_colour);
+
+			parent_marker = clutter_actor_get_parent(label_marker);
+			direction = clutter_actor_get_child_at_index(parent_marker, 0);
+			clutter_actor_set_opacity(direction, 0);
 			}
 
 		g_free(current_text);
@@ -217,10 +425,14 @@ static gboolean bar_pane_gps_create_markers_cb(gpointer data)
 	PaneGPSData *pgd = data;
 	gdouble latitude;
 	gdouble longitude;
-	ClutterActor *marker;
+	gdouble compass;
 	FileData *fd;
+	ClutterActor *parent_marker, *label_marker;
+	ClutterActor *direction;
 	ClutterColor marker_colour = { MARKER_COLOUR };
+	ClutterColor thumb_colour = { THUMB_COLOUR };
 	GString *message;
+	ClutterContent *canvas;
 
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgd->progress),
 							(gdouble)(pgd->selection_count - g_list_length(pgd->not_added)) /
@@ -237,25 +449,49 @@ static gboolean bar_pane_gps_create_markers_cb(gpointer data)
 		fd = pgd->not_added->data;
 		pgd->not_added = pgd->not_added->next;
 
-		latitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLatitude", 1000);
-		longitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLongitude", 1000);
+		latitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLatitude", 0);
+		longitude = metadata_read_GPS_coord(fd, "Xmp.exif.GPSLongitude", 0);
+		compass = metadata_read_GPS_direction(fd, "Xmp.exif.GPSImgDirection", 1000);
 
-		if ((latitude != 1000) && (longitude != 1000))
+		if (!(latitude == 0 && longitude == 0))
 			{
 			pgd->num_added++;
 
-			marker = champlain_label_new_with_text("i","courier 5", &marker_colour, &marker_colour);
+			parent_marker = champlain_marker_new();
+			clutter_actor_set_reactive(parent_marker, FALSE);
+			label_marker = champlain_label_new_with_text("i","courier 5", &marker_colour, &marker_colour);
+			clutter_actor_set_reactive(label_marker, TRUE);
+			champlain_marker_set_selection_color(&thumb_colour);
 
-			champlain_location_set_location(CHAMPLAIN_LOCATION(marker), latitude, longitude);
-			champlain_marker_layer_add_marker(pgd->icon_layer, CHAMPLAIN_MARKER(marker));
-			clutter_actor_set_reactive(marker, TRUE);
+			if (compass != 1000)
+				{
+				canvas = clutter_canvas_new();
+				clutter_canvas_set_size(CLUTTER_CANVAS (canvas), DIRECTION_SIZE, 3);
+				g_signal_connect(canvas, "draw", G_CALLBACK(bar_gps_draw_direction), NULL);
+				direction = clutter_actor_new();
+				clutter_actor_set_size(direction, DIRECTION_SIZE, 3);
+				clutter_actor_set_position(direction, 0, 0);
+				clutter_actor_set_rotation_angle(direction, CLUTTER_Z_AXIS, compass -90.00);
+				clutter_actor_set_content(direction, canvas);
+				clutter_content_invalidate(canvas);
+				g_object_unref(canvas);
 
-			g_signal_connect(G_OBJECT(marker), "button_release_event",
+				clutter_actor_add_child(parent_marker, direction);
+				clutter_actor_set_opacity(direction, 0);
+				}
+
+			clutter_actor_add_child(parent_marker, label_marker);
+
+			champlain_location_set_location(CHAMPLAIN_LOCATION(parent_marker), latitude, longitude);
+			champlain_marker_layer_add_marker(pgd->icon_layer, CHAMPLAIN_MARKER(parent_marker));
+
+			g_signal_connect(G_OBJECT(label_marker), "button_release_event",
 	 				G_CALLBACK(bar_pane_gps_marker_keypress_cb), pgd);
 
-			g_object_set_data(G_OBJECT(marker), "file_fd", fd);
+			g_object_set_data(G_OBJECT(label_marker), "file_fd", fd);
 
 			champlain_bounding_box_extend(pgd->bbox, latitude, longitude);
+
 			}
 		return TRUE;
 		}
@@ -283,13 +519,6 @@ static void bar_pane_gps_update(PaneGPSData *pgd)
 {
 	GList *list;
 
-	/* The widget does not have a parent during bar_pane_gps_new, so calling gtk_widget_show_all there gives a
-	 * "Gtk-CRITICAL **: gtk_widget_realize: assertion `GTK_WIDGET_ANCHORED (widget) || GTK_IS_INVISIBLE (widget)' failed"
-	 * error. gtk_widget_show_all can be given after it has been added to the bar.
-	 */
-	if (gtk_widget_get_parent(pgd->widget) != NULL)
-		gtk_widget_show_all(pgd->widget);
-
 	/* If a create-marker background process is running, kill it
 	 * and start again
 	 */
@@ -597,7 +826,7 @@ static GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
  */
 void bar_pane_gps_map_centreing(PaneGPSData *pgd)
 {
-	GtkWidget *dialog;
+	GenericDialog *gd;
 	GString *message = g_string_new("");
 
 	if (pgd->centre_map_checked)
@@ -611,16 +840,14 @@ void bar_pane_gps_map_centreing(PaneGPSData *pgd)
 		pgd->centre_map_checked = TRUE;
 		}
 
-	dialog = gtk_message_dialog_new(NULL,
-							  GTK_DIALOG_DESTROY_WITH_PARENT,
-							  GTK_MESSAGE_INFO,
-							  GTK_BUTTONS_CLOSE,
-							  "%s", message->str);
-	gtk_window_set_title(GTK_WINDOW(dialog), _("Map Centreing"));
-	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
-	gtk_dialog_run(GTK_DIALOG(dialog));
+	gd = generic_dialog_new(_("Map centering"),
+				"map_centering", NULL, TRUE, NULL, pgd);
+	generic_dialog_add_message(gd, GTK_STOCK_DIALOG_INFO,
+				"Map Centering", message->str, TRUE);
+	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL, NULL, TRUE);
+
+	gtk_widget_show(gd->dialog);
 
-	gtk_widget_destroy(dialog);
 	g_string_free(message, TRUE);
 }
 
@@ -628,6 +855,8 @@ static gboolean bar_pane_gps_map_keypress_cb(GtkWidget *widget, GdkEventButton *
 {
 	PaneGPSData *pgd = data;
 	GtkWidget *menu;
+	GtkClipboard *clipboard;
+	gchar *geo_coords;
 
 	if (bevent->button == MOUSE_BUTTON_RIGHT)
 		{
@@ -642,7 +871,17 @@ static gboolean bar_pane_gps_map_keypress_cb(GtkWidget *widget, GdkEventButton *
 		}
 	else if (bevent->button == MOUSE_BUTTON_LEFT)
 		{
-		return FALSE;
+		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		geo_coords = g_strdup_printf("%lf %lf",
+							champlain_view_y_to_latitude(
+								CHAMPLAIN_VIEW(pgd->gps_view),bevent->y),
+							champlain_view_x_to_longitude(
+								CHAMPLAIN_VIEW(pgd->gps_view),bevent->x));
+		gtk_clipboard_set_text(clipboard, geo_coords, -1);
+
+		g_free(geo_coords);
+
+		return TRUE;
 		}
 	else
 		{
@@ -695,6 +934,7 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 	pgd->height = height;
 
 	frame = gtk_frame_new(NULL);
+	DEBUG_NAME(frame);
 	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	gpswidget = gtk_champlain_embed_new();
@@ -709,6 +949,10 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 	gtk_scale_button_set_value(GTK_SCALE_BUTTON(slider), (gdouble)zoom);
 
 	progress = gtk_progress_bar_new();
+#if GTK_CHECK_VERSION(3,0,0)
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), "");
+	gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(progress), TRUE);
+#endif
 	state = gtk_label_new("");
 	gtk_label_set_justify(GTK_LABEL(state), GTK_JUSTIFY_CENTER);
 
@@ -751,6 +995,8 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 	g_signal_connect(pgd->gps_view, "notify::zoom-level", G_CALLBACK(bar_pane_gps_view_state_changed_cb), pgd);
 	g_signal_connect(G_OBJECT(slider), "value-changed", G_CALLBACK(bar_pane_gps_slider_changed_cb), pgd);
 
+	bar_pane_gps_dnd_init(pgd);
+
 	file_data_register_notify_func(bar_pane_gps_notify_cb, pgd, NOTIFY_PRIORITY_LOW);
 
 	pgd->create_markers_id = 0;
@@ -803,8 +1049,8 @@ GtkWidget *bar_pane_gps_new_from_config(const gchar **attribute_names, const gch
 		}
 
 	bar_pane_translate_title(PANE_COMMENT, id, &title);
-	latitude = int_latitude / 1000000;
-	longitude = int_longitude / 1000000;
+	latitude = (gdouble)int_latitude / 1000000;
+	longitude = (gdouble)int_longitude / 1000000;
 	ret = bar_pane_gps_new(id, title, map_id, zoom, latitude, longitude, expanded, height);
 	g_free(title);
 	g_free(map_id);
