@@ -38,9 +38,19 @@ typedef enum
 	DUPE_MATCH_SIM_MED  = 1 << 7,
 	DUPE_MATCH_SIM_LOW  = 1 << 8,
 	DUPE_MATCH_SIM_CUSTOM = 1 << 9,
-	DUPE_MATCH_NAME_CI = 1 << 10	/* same as name, but case insensitive */
+	DUPE_MATCH_NAME_CI = 1 << 10,	/* same as name, but case insensitive */
+	DUPE_MATCH_NAME_CONTENT = 1 << 11,	/* same name, but different content */
+	DUPE_MATCH_NAME_CI_CONTENT = 1 << 12,	/* same name - case insensitive, but different content */
+	DUPE_MATCH_ALL = 1 << 13 /* N.B. this is used as a clamp value in rcfile.c */
 } DupeMatchType;
 
+typedef enum
+{
+	DUPE_SELECT_NONE,
+	DUPE_SELECT_GROUP1,
+	DUPE_SELECT_GROUP2
+} DupeSelectType;
+
 typedef struct _DupeItem DupeItem;
 struct _DupeItem
 {
@@ -85,6 +95,11 @@ struct _DupeWindow
 	GtkWidget *status_label;
 	GtkWidget *extra_label;
 	GtkWidget *button_thumbs;
+	GtkWidget *button_rotation_invariant;
+	GtkWidget *custom_threshold;
+	GList *add_files_queue;
+	guint add_files_queue_id;
+	GtkWidget *controls_box;
 
 	gboolean show_thumbs;
 
@@ -105,6 +120,9 @@ struct _DupeWindow
 
 	ImageLoader *img_loader;
 
+	GtkTreeSortable *sortable;
+	gint set_count;
+
 	/* second set comparison stuff */
 
 	gboolean second_set;		/* second set enabled ? */
@@ -119,7 +137,7 @@ struct _DupeWindow
 };
 
 
-DupeWindow *dupe_window_new(DupeMatchType match_mask);
+DupeWindow *dupe_window_new(void);
 
 void dupe_window_clear(DupeWindow *dw);
 void dupe_window_close(DupeWindow *dw);
