@@ -815,10 +815,9 @@ static gchar *exif_build_formatted_localtime(ExifData *exif)
 				else
 					{
 					g_free(exif_date_time);
-					exif_date_time = g_strdup(tmp);
+					exif_date_time = tmp;
 					}
 				}
-				g_free(tmp);
 			}
 		putenv(time_zone_org);
 
@@ -842,7 +841,6 @@ static gchar *exif_build_formatted_localtime(ExifData *exif)
  */
 static gchar *exif_build_formatted_timezone(ExifData *exif)
 {
-	gchar *time_zone = NULL;
 	gchar *exif_date_time = NULL;
 	gchar *timezone = NULL;
 	gchar *countryname = NULL;
@@ -944,6 +942,8 @@ ExifFormattedText ExifFormattedList[] = {
 	{"file.group",				N_("File group"), 	NULL},
 	{"file.link",				N_("File link"), 	NULL},
 	{"file.class",				N_("File class"), 	NULL},
+	{"file.page_no",			N_("Page no."), 	NULL},
+	{"lua.lensID",				N_("Lens"), 		NULL},
 	{ NULL, NULL, NULL }
 };
 
@@ -969,8 +969,7 @@ gchar *exif_get_description_by_key(const gchar *key)
 {
 	if (!key) return NULL;
 
-	if (strncmp(key, EXIF_FORMATTED(), EXIF_FORMATTED_LEN) == 0 ||
-	    strncmp(key, "file.", 5) == 0)
+	if (strncmp(key, EXIF_FORMATTED(), EXIF_FORMATTED_LEN) == 0 || strncmp(key, "file.", 5) == 0 || strncmp(key, "lua.", 4) == 0)
 		{
 		gint i;
 
@@ -1192,6 +1191,8 @@ static gchar *mode_number(mode_t m)
 
 gchar *metadata_file_info(FileData *fd, const gchar *key, MetadataFormat format)
 {
+	gchar *page_n_of_m;
+
 	if (strcmp(key, "file.size") == 0)
 		{
 		return g_strdup_printf("%ld", (long)fd->size);
@@ -1224,6 +1225,18 @@ gchar *metadata_file_info(FileData *fd, const gchar *key, MetadataFormat format)
 		{
 		return g_strdup(fd->sym_link);
 		}
+	if (strcmp(key, "file.page_no") == 0)
+		{
+		if (fd->page_total > 1)
+			{
+			page_n_of_m = g_strdup_printf("[%d/%d]", fd->page_num + 1, fd->page_total);
+			return page_n_of_m;
+			}
+		else
+			{
+			return NULL;
+			}
+		}
 	return g_strdup("");
 }
 
