@@ -435,6 +435,8 @@ static FileData *file_data_new(const gchar *path_utf8, struct stat *st, gboolean
 	fd->exifdate = 0;
 	fd->rating = STAR_RATING_NOT_READ;
 	fd->format_class = filter_file_get_class(path_utf8);
+	fd->page_num = 0;
+	fd->page_total = 0;
 
 	user = getpwuid(st->st_uid);
 	if (!user)
@@ -581,6 +583,10 @@ void read_rating_data(FileData *file)
 		file->rating = atoi(rating_str);
 		g_free(rating_str);
 		}
+	else
+		{
+		file->rating = 0;
+		}
 }
 
 void set_exif_time_data(GList *files)
@@ -1888,6 +1894,46 @@ GList *file_data_filter_file_filter_list(GList *list, GRegex *filter)
 	return list;
 }
 
+static gboolean file_data_filter_class(FileData *fd, guint filter)
+{
+	gint i;
+
+	for (i = 0; i < FILE_FORMAT_CLASSES; i++)
+		{
+		if (filter & (1 << i))
+			{
+			if ((FileFormatClass)i == filter_file_get_class(fd->path))
+				{
+				return TRUE;
+				}
+			}
+		}
+
+	return FALSE;
+}
+
+GList *file_data_filter_class_list(GList *list, guint filter)
+{
+	GList *work;
+
+	work = list;
+	while (work)
+		{
+		FileData *fd = work->data;
+		GList *link = work;
+		work = work->next;
+
+		if (!file_data_filter_class(fd, filter))
+			{
+			list = g_list_remove_link(list, link);
+			file_data_unref(fd);
+			g_list_free(link);
+			}
+		}
+
+	return list;
+}
+
 static void file_data_notify_mark_func(gpointer key, gpointer value, gpointer user_data)
 {
 	FileData *fd = value;
@@ -3317,7 +3363,7 @@ gboolean marks_list_load(const gchar *path)
 			marks_value = strtok(NULL, ",");
 			if (isfile(file_path))
 				{
-				FileData *fd = file_data_new_group(file_path);
+				FileData *fd = file_data_new_no_grouping(file_path);
 				file_data_ref(fd);
 				gint n = 0;
 				while (n <= 9)
@@ -3394,4 +3440,49 @@ void marks_clear_all()
 {
 	g_hash_table_foreach(file_data_pool, marks_clear, NULL);
 }
+
+void file_data_set_page_num(FileData *fd, gint page_num)
+{
+	if (fd->page_total > 1 && page_num < 0)
+		{
+		fd->page_num = fd->page_total - 1;
+		}
+	else if (fd->page_total > 1 && page_num <= fd->page_total)
+		{
+		fd->page_num = page_num - 1;
+		}
+	else
+		{
+		fd->page_num = 0;
+		}
+	file_data_send_notification(fd, NOTIFY_REREAD);
+}
+
+void file_data_inc_page_num(FileData *fd)
+{
+	if (fd->page_total > 0 && fd->page_num < fd->page_total - 1)
+		{
+		fd->page_num = fd->page_num + 1;
+		}
+	else if (fd->page_total == 0)
+		{
+		fd->page_num = fd->page_num + 1;
+		}
+	file_data_send_notification(fd, NOTIFY_REREAD);
+}
+
+void file_data_dec_page_num(FileData *fd)
+{
+	if (fd->page_num > 0)
+		{
+		fd->page_num = fd->page_num - 1;
+		}
+	file_data_send_notification(fd, NOTIFY_REREAD);
+}
+
+void file_data_set_page_total(FileData *fd, gint page_total)
+{
+	fd->page_total = page_total;
+}
+
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
