@@ -103,6 +103,8 @@ GList *file_data_filter_marks_list(GList *list, guint filter);
 gboolean file_data_filter_file_filter(FileData *fd, GRegex *filter);
 GList *file_data_filter_file_filter_list(GList *list, GRegex *filter);
 
+GList *file_data_filter_class_list(GList *list, guint filter);
+
 gint file_data_get_user_orientation(FileData *fd);
 void file_data_set_user_orientation(FileData *fd, gint value);
 
@@ -172,5 +174,10 @@ gboolean marks_list_save(gchar *path, gboolean clear);
 gboolean marks_list_load(const gchar *path);
 void marks_clear_all();
 void read_rating_data(FileData *file);
+
+void file_data_inc_page_num(FileData *fd);
+void file_data_dec_page_num(FileData *fd);
+void file_data_set_page_total(FileData *fd, gint page_total);
+void file_data_set_page_num(FileData *fd, gint page_num);
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
