@@ -21,13 +21,20 @@
 
 #include "main.h"
 #include "image-load.h"
+#include "image_load_cr3.h"
 #include "image_load_gdk.h"
 #include "image_load_jpeg.h"
 #include "image_load_tiff.h"
 #include "image_load_dds.h"
+#include "image_load_djvu.h"
 #include "image_load_pdf.h"
+#include "image_load_psd.h"
+#include "image_load_heif.h"
 #include "image_load_ffmpegthumbnailer.h"
 #include "image_load_collection.h"
+#include "image_load_webp.h"
+#include "image_load_j2k.h"
+#include "image_load_svgz.h"
 
 #include "exif.h"
 #include "filedata.h"
@@ -57,6 +64,7 @@ enum {
 static guint signals[SIGNAL_COUNT] = { 0 };
 
 static void image_loader_init(GTypeInstance *instance, gpointer g_class);
+static void image_loader_class_init_wrapper(void *data, void *user_data);
 static void image_loader_class_init(ImageLoaderClass *class);
 static void image_loader_finalize(GObject *object);
 static void image_loader_stop(ImageLoader *il);
@@ -70,7 +78,7 @@ GType image_loader_get_type(void)
 			sizeof(ImageLoaderClass),
 			NULL,   /* base_init */
 			NULL,   /* base_finalize */
-			(GClassInitFunc)image_loader_class_init, /* class_init */
+			(GClassInitFunc)image_loader_class_init_wrapper, /* class_init */
 			NULL,   /* class_finalize */
 			NULL,   /* class_data */
 			sizeof(ImageLoader),
@@ -124,6 +132,11 @@ static void image_loader_init(GTypeInstance *instance, gpointer g_class)
 	DEBUG_1("new image loader %p, bufsize=%" G_GSIZE_FORMAT " idle_loop=%u", il, il->read_buffer_size, il->idle_read_loop_count);
 }
 
+static void image_loader_class_init_wrapper(void *data, void *user_data)
+{
+	image_loader_class_init(data);
+}
+
 static void image_loader_class_init(ImageLoaderClass *class)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (class);
@@ -610,6 +623,10 @@ static void image_loader_stop_loader(ImageLoader *il)
 
 static void image_loader_setup_loader(ImageLoader *il)
 {
+#if defined HAVE_TIFF || defined HAVE_PDF || defined HAVE_HEIF || defined HAVE_DJVU
+	gchar *format;
+#endif
+
 	g_mutex_lock(il->data_mutex);
 #ifdef HAVE_FFMPEGTHUMBNAILER
 	if (il->fd->format_class == FORMAT_CLASS_VIDEO)
@@ -620,13 +637,45 @@ static void image_loader_setup_loader(ImageLoader *il)
 	else
 #endif
 #ifdef HAVE_PDF
-	if (il->fd->format_class == FORMAT_CLASS_PDF)
+	if (il->bytes_total >= 4 &&
+	    (memcmp(il->mapped_file + 0, "%PDF", 4) == 0))
 		{
 		DEBUG_1("Using custom pdf loader");
 		image_loader_backend_set_pdf(&il->backend);
 		}
 	else
 #endif
+#ifdef HAVE_HEIF
+	if (il->bytes_total >= 12 &&
+		((memcmp(il->mapped_file + 4, "ftypheic", 8) == 0) ||
+		(memcmp(il->mapped_file + 4, "ftypmsf1", 8) == 0) ||
+		(memcmp(il->mapped_file + 4, "ftypmif1", 8) == 0)))
+		{
+		DEBUG_1("Using custom heif loader");
+		image_loader_backend_set_heif(&il->backend);
+		}
+	else
+#endif
+#ifdef HAVE_WEBP
+	if (il->bytes_total >= 12 &&
+		(memcmp(il->mapped_file, "RIFF", 4) == 0) &&
+		(memcmp(il->mapped_file + 8, "WEBP", 4) == 0))
+		{
+		DEBUG_1("Using custom webp loader");
+		image_loader_backend_set_webp(&il->backend);
+		}
+	else
+#endif
+#ifdef HAVE_DJVU
+	if (il->bytes_total >= 16 &&
+		(memcmp(il->mapped_file, "AT&TFORM", 8) == 0) &&
+		(memcmp(il->mapped_file + 12, "DJV", 3) == 0))
+		{
+		DEBUG_1("Using custom djvu loader");
+		image_loader_backend_set_djvu(&il->backend);
+		}
+	else
+#endif
 #ifdef HAVE_JPEG
 	if (il->bytes_total >= 2 && il->mapped_file[0] == 0xff && il->mapped_file[1] == 0xd8)
 		{
@@ -634,6 +683,14 @@ static void image_loader_setup_loader(ImageLoader *il)
 		image_loader_backend_set_jpeg(&il->backend);
 		}
 	else
+	if (il->bytes_total >= 11 &&
+		(memcmp(il->mapped_file + 4, "ftypcrx", 7) == 0) &&
+		(memcmp(il->mapped_file + 64, "CanonCR3", 8) == 0))
+		{
+		DEBUG_1("Using custom cr3 loader");
+		image_loader_backend_set_cr3(&il->backend);
+		}
+	else
 #endif
 #ifdef HAVE_TIFF
 	if (il->bytes_total >= 10 &&
@@ -653,15 +710,74 @@ static void image_loader_setup_loader(ImageLoader *il)
 		image_loader_backend_set_dds(&il->backend);
 		}
 	else
+	if (il->bytes_total >= 6 &&
+		(memcmp(il->mapped_file, "8BPS\0\x01", 6) == 0))
+		{
+		DEBUG_1("Using custom psd loader");
+		image_loader_backend_set_psd(&il->backend);
+		}
+	else
+#ifdef HAVE_J2K
+	if (il->bytes_total >= 12 &&
+		(memcmp(il->mapped_file, "\0\0\0\x0CjP\x20\x20\x0D\x0A\x87\x0A", 12) == 0))
+		{
+		DEBUG_1("Using custom j2k loader");
+		image_loader_backend_set_j2k(&il->backend);
+		}
+	else
+#endif
 	if (il->fd->format_class == FORMAT_CLASS_COLLECTION)
 		{
 		DEBUG_1("Using custom collection loader");
 		image_loader_backend_set_collection(&il->backend);
 		}
+	else
+	if (g_strcmp0(strrchr(il->fd->path, '.'), ".svgz") == 0)
+		{
+		DEBUG_1("Using custom svgz loader");
+		image_loader_backend_set_svgz(&il->backend);
+		}
 	else
 		image_loader_backend_set_default(&il->backend);
 
 	il->loader = il->backend.loader_new(image_loader_area_updated_cb, image_loader_size_cb, image_loader_area_prepared_cb, il);
+
+#ifdef HAVE_TIFF
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "tiff") == 0)
+		{
+		il->backend.set_page_num(il->loader, il->fd->page_num);
+		}
+	g_free(format);
+#endif
+
+#ifdef HAVE_PDF
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "pdf") == 0)
+		{
+		il->backend.set_page_num(il->loader, il->fd->page_num);
+		}
+	g_free(format);
+#endif
+
+#ifdef HAVE_HEIF
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "heif") == 0)
+		{
+		il->backend.set_page_num(il->loader, il->fd->page_num);
+		}
+	g_free(format);
+#endif
+
+#ifdef HAVE_DJVU
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "djvu") == 0)
+		{
+		il->backend.set_page_num(il->loader, il->fd->page_num);
+		}
+	g_free(format);
+#endif
+
 	g_mutex_unlock(il->data_mutex);
 }
 
@@ -721,6 +837,9 @@ static gboolean image_loader_continue(ImageLoader *il)
 
 static gboolean image_loader_begin(ImageLoader *il)
 {
+#if defined HAVE_TIFF || defined HAVE_PDF || defined HAVE_HEIF || defined HAVE_DJVU
+	gchar *format;
+#endif
 	gssize b;
 
 	if (il->pixbuf) return FALSE;
@@ -745,6 +864,43 @@ static gboolean image_loader_begin(ImageLoader *il)
 		return FALSE;
 		}
 
+#ifdef HAVE_PDF
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "pdf") == 0)
+		{
+		gint i = il->backend.get_page_total(il->loader);
+		file_data_set_page_total(il->fd, i);
+		}
+	g_free(format);
+#endif
+#ifdef HAVE_HEIF
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "heif") == 0)
+		{
+		gint i = il->backend.get_page_total(il->loader);
+		file_data_set_page_total(il->fd, i);
+		}
+	g_free(format);
+#endif
+#ifdef HAVE_DJVU
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "djvu") == 0)
+		{
+		gint i = il->backend.get_page_total(il->loader);
+		file_data_set_page_total(il->fd, i);
+		}
+	g_free(format);
+#endif
+#ifdef HAVE_TIFF
+	format = il->backend.get_format_name(il->loader);
+	if (g_strcmp0(format, "tiff") == 0)
+		{
+		gint i = il->backend.get_page_total(il->loader);
+		file_data_set_page_total(il->fd, i);
+		}
+	g_free(format);
+#endif
+
 	il->bytes_read += b;
 
 	/* read until size is known */
