@@ -29,6 +29,7 @@
 #include "exif.h"
 #include "metadata.h"
 #include "histogram.h"
+#include "history_list.h"
 #include "image-load.h"
 #include "image-overlay.h"
 #include "layout.h"
@@ -44,7 +45,7 @@
 
 static GList *image_list = NULL;
 
-static void image_update_title(ImageWindow *imd);
+void image_update_title(ImageWindow *imd);
 static void image_read_ahead_start(ImageWindow *imd);
 static void image_cache_set(ImageWindow *imd, FileData *fd);
 
@@ -332,7 +333,7 @@ static void image_zoom_cb(PixbufRenderer *pr, gdouble zoom, gpointer data)
  *-------------------------------------------------------------------
  */
 
-static void image_update_title(ImageWindow *imd)
+void image_update_title(ImageWindow *imd)
 {
 	gchar *title = NULL;
 	gchar *zoom = NULL;
@@ -564,7 +565,7 @@ static void image_post_process_tile_color_cb(PixbufRenderer *pr, GdkPixbuf **pix
 	ImageWindow *imd = (ImageWindow *)data;
 	if (imd->cm) color_man_correct_region(imd->cm, *pixbuf, x, y, w, h);
 	if (imd->desaturate) pixbuf_desaturate_rect(*pixbuf, x, y, w, h);
-
+	if (imd->overunderexposed) pixbuf_highlight_overunderexposed(*pixbuf, x, y, w, h);
 }
 
 void image_alter_orientation(ImageWindow *imd, FileData *fd_n, AlterType type)
@@ -658,7 +659,7 @@ void image_alter_orientation(ImageWindow *imd, FileData *fd_n, AlterType type)
 void image_set_desaturate(ImageWindow *imd, gboolean desaturate)
 {
 	imd->desaturate = desaturate;
-	if (imd->cm || imd->desaturate)
+	if (imd->cm || imd->desaturate || imd->overunderexposed)
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, image_post_process_tile_color_cb, (gpointer) imd, (imd->cm != NULL) );
 	else
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, NULL, NULL, TRUE);
@@ -670,6 +671,26 @@ gboolean image_get_desaturate(ImageWindow *imd)
 	return imd->desaturate;
 }
 
+void image_set_overunderexposed(ImageWindow *imd, gboolean overunderexposed)
+{
+	imd->overunderexposed = overunderexposed;
+	if (imd->cm || imd->desaturate || imd->overunderexposed)
+		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, image_post_process_tile_color_cb, (gpointer) imd, (imd->cm != NULL) );
+	else
+		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, NULL, NULL, TRUE);
+	pixbuf_renderer_set_orientation((PixbufRenderer *)imd->pr, imd->orientation);
+}
+
+gboolean image_get_overunderexposed(ImageWindow *imd)
+{
+	return imd->overunderexposed;
+}
+
+void image_set_ignore_alpha(ImageWindow *imd, gboolean ignore_alpha)
+{
+   pixbuf_renderer_set_ignore_alpha((PixbufRenderer *)imd->pr, ignore_alpha);
+}
+
 /*
  *-------------------------------------------------------------------
  * read ahead (prebuffer)
@@ -856,10 +877,11 @@ static void image_load_done_cb(ImageLoader *il, gpointer data)
 				break;
 			case FORMAT_CLASS_VIDEO:
 				pixbuf = pixbuf_inline(PIXBUF_INLINE_VIDEO);
+				break;
 			case FORMAT_CLASS_COLLECTION:
 				pixbuf = pixbuf_inline(PIXBUF_INLINE_COLLECTION);
 				break;
-			case FORMAT_CLASS_PDF:
+			case FORMAT_CLASS_DOCUMENT:
 				pixbuf = pixbuf_inline(PIXBUF_INLINE_ICON_PDF);
 				break;
 			default:
@@ -1136,12 +1158,47 @@ static gboolean image_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, gpoin
 static gboolean image_scroll_cb(GtkWidget *widget, GdkEventScroll *event, gpointer data)
 {
 	ImageWindow *imd = data;
+	gboolean in_lw = FALSE;
+	gint i = 0;
+	LayoutWindow *lw = NULL;
 
-	if (imd->func_scroll &&
-	    event && event->type == GDK_SCROLL)
+	if (imd->func_scroll && event && event->type == GDK_SCROLL)
 		{
-		imd->func_scroll(imd, event, imd->data_scroll);
-		return TRUE;
+		layout_valid(&lw);
+		/* check if the image is in a layout window */
+		for (i = 0; i < MAX_SPLIT_IMAGES; i++)
+			{
+			if (imd == lw->split_images[i])
+				{
+				in_lw = TRUE;
+				break;
+				}
+			}
+
+		if (in_lw)
+			{
+			if (lw->options.split_pane_sync)
+				{
+				for (i = 0; i < MAX_SPLIT_IMAGES; i++)
+					{
+					if (lw->split_images[i])
+						{
+						layout_image_activate(lw, i, FALSE);
+						imd->func_scroll(lw->split_images[i], event, lw->split_images[i]->data_scroll);
+						}
+					}
+				}
+			else
+				{
+				imd->func_scroll(imd, event, imd->data_scroll);
+				}
+			return TRUE;
+			}
+		else
+			{
+			imd->func_scroll(imd, event, imd->data_scroll);
+			return TRUE;
+			}
 		}
 
 	return FALSE;
@@ -1349,7 +1406,7 @@ void image_change_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gdouble zoom, gboo
 		image_post_process_color(imd, 0, FALSE); /* TODO: error handling */
 		}
 
-	if (imd->cm || imd->desaturate)
+	if (imd->cm || imd->desaturate || imd->overunderexposed)
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, image_post_process_tile_color_cb, (gpointer) imd, (imd->cm != NULL) );
 
 	image_state_set(imd, IMAGE_STATE_IMAGE);
@@ -1369,6 +1426,11 @@ void image_change_from_collection(ImageWindow *imd, CollectionData *cd, CollectI
 		collection_table_unselect_all(cw->table);
 		collection_table_select(cw->table,info);
 		}
+
+	if (info->fd)
+		{
+		image_chain_append_end(info->fd->path);
+		}
 }
 
 CollectionData *image_get_collection(ImageWindow *imd, CollectInfo **info)
@@ -1467,7 +1529,7 @@ void image_move_from_image(ImageWindow *imd, ImageWindow *source)
 
 	pixbuf_renderer_move(PIXBUF_RENDERER(imd->pr), PIXBUF_RENDERER(source->pr));
 
-	if (imd->cm || imd->desaturate)
+	if (imd->cm || imd->desaturate || imd->overunderexposed)
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, image_post_process_tile_color_cb, (gpointer) imd, (imd->cm != NULL) );
 	else
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, NULL, NULL, TRUE);
@@ -1529,7 +1591,7 @@ void image_copy_from_image(ImageWindow *imd, ImageWindow *source)
 
 	pixbuf_renderer_copy(PIXBUF_RENDERER(imd->pr), PIXBUF_RENDERER(source->pr));
 
-	if (imd->cm || imd->desaturate)
+	if (imd->cm || imd->desaturate || imd->overunderexposed)
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, image_post_process_tile_color_cb, (gpointer) imd, (imd->cm != NULL) );
 	else
 		pixbuf_renderer_set_post_process_func((PixbufRenderer *)imd->pr, NULL, NULL, TRUE);
