@@ -88,6 +88,9 @@ void image_set_scroll_center(ImageWindow *imd, gdouble x, gdouble y);
 void image_alter_orientation(ImageWindow *imd, FileData *fd, AlterType type);
 void image_set_desaturate(ImageWindow *imd, gboolean desaturate);
 gboolean image_get_desaturate(ImageWindow *imd);
+void image_set_overunderexposed(ImageWindow *imd, gboolean overunderexposed);
+gboolean image_get_overunderexposed(ImageWindow *imd);
+void image_set_ignore_alpha(ImageWindow *imd, gboolean ignore_alpha);
 
 /* zoom */
 void image_zoom_adjust(ImageWindow *imd, gdouble increment);
@@ -151,6 +154,6 @@ void image_set_image_as_tiles(ImageWindow *imd, gint width, gint height,
 void image_options_sync(void);
 
 void image_get_rectangle(gint *x1, gint *y1, gint *x2, gint *y2);
-
+void image_update_title(ImageWindow *imd);
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
