@@ -52,6 +52,11 @@
 #ifdef HAVE_LIRC
 #include "lirc.h"
 #endif
+#ifdef GDK_WINDOWING_X11
+#if GTK_CHECK_VERSION(3,10,0)
+#include <gdk/gdkx.h>
+#endif
+#endif
 
 #define MAINWINDOW_DEF_WIDTH 700
 #define MAINWINDOW_DEF_HEIGHT 500
@@ -150,6 +155,23 @@ LayoutWindow *layout_find_by_layout_id(const gchar *id)
 	return NULL;
 }
 
+gchar *layout_get_unique_id()
+{
+	char id[10];
+	gint i;
+
+	i = 1;
+	while (TRUE)
+		{
+		g_snprintf(id, sizeof(id), "lw%d", i);
+		if (!layout_find_by_layout_id(id))
+			{
+			return g_strdup(id);
+			}
+		i++;
+		}
+}
+
 static void layout_set_unique_id(LayoutWindow *lw)
 {
 	char id[10];
@@ -159,12 +181,6 @@ static void layout_set_unique_id(LayoutWindow *lw)
 	g_free(lw->options.id);
 	lw->options.id = NULL;
 
-	if (!layout_find_by_layout_id("main"))
-		{
-		lw->options.id = g_strdup("main");
-		return;
-		}
-
 	i = 1;
 	while (TRUE)
 		{
@@ -262,9 +278,13 @@ static void layout_path_entry_cb(const gchar *path, gpointer data)
 	gchar *buf;
 
 	buf = g_strdup(path);
-	parse_out_relatives(buf);
 
-	layout_set_path(lw, buf);
+	if (!download_web_file(buf, FALSE, lw))
+		{
+		parse_out_relatives(buf);
+
+		layout_set_path(lw, buf);
+		}
 
 	g_free(buf);
 }
@@ -289,7 +309,6 @@ static void layout_path_entry_tab_append_cb(const gchar *path, gpointer data, gi
 
 static gboolean path_entry_tooltip_cb(GtkWidget *widget, gpointer data)
 {
-	LayoutWindow *lw = data;
 	GList *box_child_list;
 	GtkComboBox *path_entry;
 	gchar *current_path;
@@ -305,6 +324,22 @@ static gboolean path_entry_tooltip_cb(GtkWidget *widget, gpointer data)
 	return FALSE;
 }
 
+void show_menu_scrollbar_cb(GtkWidget *widget, GdkRectangle *allocation, gpointer data)
+{
+	GtkRequisition requisition;
+
+	gtk_widget_size_request(widget, &requisition);
+
+	if (allocation->width <= requisition.width)
+		{
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(data), GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
+		}
+	else
+		{
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(data), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+		}
+}
+
 static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 {
 	GtkWidget *box;
@@ -312,14 +347,37 @@ static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 	GtkWidget *scd;
 	GtkWidget *menu_tool_bar;
 	GtkWidget *tabcomp;
+	GtkWidget *menu_bar;
+	GtkWidget *toolbar;
+	GtkWidget *scroll_window;
 
 	box = gtk_vbox_new(FALSE, 0);
 
-	menu_tool_bar = layout_actions_menu_tool_bar(lw);
-	DEBUG_NAME(menu_tool_bar);
-	gtk_widget_show(menu_tool_bar);
-	gtk_box_pack_start(GTK_BOX(lw->main_box), lw->menu_tool_bar, FALSE, FALSE, 0);
-	if (lw->options.toolbar_hidden) gtk_widget_hide(lw->toolbar[TOOLBAR_MAIN]);
+	if (!options->expand_menu_toolbar)
+		{
+		menu_bar = layout_actions_menu_bar(lw);
+
+		toolbar = layout_actions_toolbar(lw, TOOLBAR_MAIN);
+		scroll_window = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_window), GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
+		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll_window), menu_bar);
+		g_signal_connect(G_OBJECT(menu_bar), "size-allocate", G_CALLBACK(show_menu_scrollbar_cb), scroll_window);
+
+		gtk_widget_show(scroll_window);
+		gtk_widget_show(menu_bar);
+		if (lw->options.toolbar_hidden) gtk_widget_hide(toolbar);
+
+		gtk_box_pack_start(GTK_BOX(box), scroll_window, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(box), toolbar, FALSE, FALSE, 0);
+		}
+	else
+		{
+		menu_tool_bar = layout_actions_menu_tool_bar(lw);
+		DEBUG_NAME(menu_tool_bar);
+		gtk_widget_show(menu_tool_bar);
+		gtk_box_pack_start(GTK_BOX(lw->main_box), lw->menu_tool_bar, FALSE, FALSE, 0);
+		if (lw->options.toolbar_hidden) gtk_widget_hide(lw->toolbar[TOOLBAR_MAIN]);
+		}
 
 	tabcomp = tab_completion_new_with_history(&lw->path_entry, NULL, "path_list", -1,
 						  layout_path_entry_cb, lw);
@@ -343,8 +401,8 @@ static GtkWidget *layout_tool_setup(LayoutWindow *lw)
 	DEBUG_NAME(box_folders);
 	gtk_box_pack_start(GTK_BOX(box), box_folders, TRUE, TRUE, 0);
 
-	lw->vd = vd_new(lw->options.dir_view_type, lw->dir_fd);
-	vd_set_layout(lw->vd, lw);
+	lw->vd = vd_new(lw);
+
 	vd_set_select_func(lw->vd, layout_vd_select_cb, lw);
 
 	lw->dir_view = lw->vd->widget;
@@ -575,7 +633,6 @@ static GtkWidget *layout_zoom_button(LayoutWindow *lw, GtkWidget *box, gint size
 
 void layout_status_update_progress(LayoutWindow *lw, gdouble val, const gchar *text)
 {
-	static gdouble thumb = 0;
 	static gdouble meta = 0;
 
 	if (!layout_valid(&lw)) return;
@@ -585,7 +642,6 @@ void layout_status_update_progress(LayoutWindow *lw, gdouble val, const gchar *t
 	 */
 	if(!g_strcmp0(text, "Loading thumbs..."))
 		{
-		thumb = val;
 		if (meta)
 			{
 			return;
@@ -676,6 +732,9 @@ void layout_status_update_info(LayoutWindow *lw, const gchar *text)
 void layout_status_update_image(LayoutWindow *lw)
 {
 	guint64 n;
+	FileData *fd;
+	gint page_total;
+	gint page_num;
 
 	if (!layout_valid(&lw) || !lw->image) return;
 	if (!lw->info_zoom || !lw->info_details) return; /*called from layout_style_set */
@@ -712,10 +771,19 @@ void layout_status_update_image(LayoutWindow *lw)
 		else
 			{
 			gint width, height;
-
+			fd = image_get_fd(lw->image);
+			page_total = fd->page_total;
+			page_num = fd->page_num + 1;
 			image_get_image_size(lw->image, &width, &height);
-			text = g_strdup_printf(_("( %d x %d ) %s bytes"),
-					       width, height, b);
+
+			if (page_total > 1)
+				{
+				text = g_strdup_printf(_("( %d x %d ) %s bytes %s%d%s%d%s"), width, height, b, "[", page_num, "/", page_total, "]");
+				}
+			else
+				{
+				text = g_strdup_printf(_("( %d x %d ) %s bytes"), width, height, b);
+				}
 			}
 
 		g_signal_emit_by_name (lw->image->pr, "update-pixel");
@@ -822,7 +890,7 @@ static void layout_status_setup(LayoutWindow *lw, GtkWidget *box, gboolean small
 		}
 	lw->info_details = layout_status_label(NULL, hbox, TRUE, 0, TRUE);
 	DEBUG_NAME(lw->info_details);
-	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_details), _("(Image dimensions) Image size"));
+	gtk_widget_set_tooltip_text(GTK_WIDGET(lw->info_details), _("(Image dimensions) Image size [page n of m]"));
 	toolbar = layout_actions_toolbar(lw, TOOLBAR_STATUS);
 
 	toolbar_frame = gtk_frame_new(NULL);
@@ -1328,6 +1396,18 @@ void layout_views_set(LayoutWindow *lw, DirViewType dir_view_type, FileViewType
 	layout_style_set(lw, -1, NULL);
 }
 
+void layout_views_set_sort(LayoutWindow *lw, SortType method, gboolean ascend)
+{
+	if (!layout_valid(&lw)) return;
+
+	if (lw->options.dir_view_list_sort.method == method && lw->options.dir_view_list_sort.ascend == ascend) return;
+
+	lw->options.dir_view_list_sort.method = method;
+	lw->options.dir_view_list_sort.ascend = ascend;
+
+	layout_style_set(lw, -1, NULL);
+}
+
 gboolean layout_views_get(LayoutWindow *lw, DirViewType *dir_view_type, FileViewType *file_view_type)
 {
 	if (!layout_valid(&lw)) return FALSE;
@@ -1533,7 +1613,7 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 
 		gtk_window_set_resizable(GTK_WINDOW(lw->tools), TRUE);
 		gtk_container_set_border_width(GTK_CONTAINER(lw->tools), 0);
-		gtk_container_remove(GTK_CONTAINER(lw->main_box), lw->menu_tool_bar);
+		if (options->expand_menu_toolbar) gtk_container_remove(GTK_CONTAINER(lw->main_box), lw->menu_tool_bar);
 
 		new_window = TRUE;
 		}
@@ -1549,7 +1629,7 @@ static void layout_tools_setup(LayoutWindow *lw, GtkWidget *tools, GtkWidget *fi
 	vbox = gtk_vbox_new(FALSE, 0);
 	DEBUG_NAME(vbox);
 	gtk_container_add(GTK_CONTAINER(lw->tools), vbox);
-	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(lw->menu_tool_bar), FALSE, FALSE, 0);
+	if (options->expand_menu_toolbar) gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(lw->menu_tool_bar), FALSE, FALSE, 0);
 	gtk_widget_show(vbox);
 
 	layout_status_setup(lw, vbox, TRUE);
@@ -1702,7 +1782,14 @@ static void layout_grid_setup(LayoutWindow *lw)
 
 	lw->group_box = gtk_vbox_new(FALSE, 0);
 	DEBUG_NAME(lw->group_box);
-	gtk_box_pack_end(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
+	if (options->expand_menu_toolbar)
+		{
+		gtk_box_pack_end(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
+		}
+	else
+		{
+		gtk_box_pack_start(GTK_BOX(lw->main_box), lw->group_box, TRUE, TRUE, 0);
+		}
 	gtk_widget_show(lw->group_box);
 
 	priority_location = layout_grid_compass(lw);
@@ -1843,9 +1930,18 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	/* lw->image is preserved together with lw->utility_box */
 	if (lw->utility_box) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->utility_box)), lw->utility_box);
 
-	if (lw->toolbar[TOOLBAR_STATUS]) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->toolbar[TOOLBAR_STATUS])), lw->toolbar[TOOLBAR_STATUS]);
+	if (options->expand_menu_toolbar)
+		{
+		if (lw->toolbar[TOOLBAR_STATUS]) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->toolbar[TOOLBAR_STATUS])), lw->toolbar[TOOLBAR_STATUS]);
 
-	if (lw->menu_tool_bar) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->menu_tool_bar)), lw->menu_tool_bar);
+		if (lw->menu_tool_bar) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->menu_tool_bar)), lw->menu_tool_bar);
+		}
+	else
+		{
+		if (lw->menu_bar) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->menu_bar)), lw->menu_bar);
+			for (i = 0; i < TOOLBAR_COUNT; i++)
+				if (lw->toolbar[i]) gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(lw->toolbar[i])), lw->toolbar[i]);	
+		}
 
 	/* clear it all */
 
@@ -2246,9 +2342,6 @@ void layout_show_config_window(LayoutWindow *lw)
 	pref_checkbox_new_int(group, _("Show date in directories list view"),
 			      lc->options.show_directory_date, &lc->options.show_directory_date);
 
-	pref_checkbox_new_int(group, _("Exit program when this window is closed"),
-			      lc->options.exit_on_close, &lc->options.exit_on_close);
-
 	group = pref_group_new(vbox, FALSE, _("Start-up directory:"), GTK_ORIENTATION_VERTICAL);
 
 	button = pref_radiobutton_new(group, NULL, _("No change"),
@@ -2281,6 +2374,12 @@ void layout_show_config_window(LayoutWindow *lw)
 void layout_sync_options_with_current_state(LayoutWindow *lw)
 {
 	Histogram *histogram;
+#ifdef GDK_WINDOWING_X11
+#if GTK_CHECK_VERSION(3,10,0)
+	GdkWindow *window;
+#endif
+#endif
+
 	if (!layout_valid(&lw)) return;
 
 	lw->options.main_window.maximized =  window_maximized(lw->window);
@@ -2309,6 +2408,13 @@ void layout_sync_options_with_current_state(LayoutWindow *lw)
 	layout_geometry_get_log_window(lw, &lw->options.log_window.x, &lw->options.log_window.y,
 	                                 &lw->options.log_window.w, &lw->options.log_window.h);
 
+#ifdef GDK_WINDOWING_X11
+#if GTK_CHECK_VERSION(3,10,0)
+	window = gtk_widget_get_window(GTK_WIDGET(lw->window));
+	lw->options.workspace = gdk_x11_window_get_desktop(window);
+#endif
+#endif
+	return;
 }
 
 void layout_apply_options(LayoutWindow *lw, LayoutOptions *lop)
@@ -2328,11 +2434,30 @@ void layout_apply_options(LayoutWindow *lw, LayoutOptions *lop)
 	if (refresh_lists) layout_refresh(lw);
 }
 
+void save_layout(LayoutWindow *lw)
+{
+	gchar *path;
+	gchar *xml_name;
+
+	if (!g_str_has_prefix(lw->options.id, "lw"))
+		{
+		xml_name = g_strdup_printf("%s.xml", lw->options.id);
+		path = g_build_filename(get_window_layouts_dir(), xml_name, NULL);
+		save_config_to_file(path, options, lw);
+
+		g_free(xml_name);
+		g_free(path);
+		}
+}
 
 void layout_close(LayoutWindow *lw)
 {
-	if (!lw->options.exit_on_close && layout_window_list && layout_window_list->next)
+	GList *list;
+	LayoutWindow *tmp_lw;
+
+	if (layout_window_list && layout_window_list->next)
 		{
+		save_layout(lw);
 		layout_free(lw);
 		}
 	else
@@ -2390,6 +2515,29 @@ LayoutWindow *layout_new(FileData *dir_fd, LayoutOptions *lop)
 	return layout_new_with_geometry(dir_fd, lop, NULL);
 }
 
+gboolean release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+	return defined_mouse_buttons(widget, event, data);
+}
+
+#ifdef GDK_WINDOWING_X11
+#if GTK_CHECK_VERSION(3,10,0)
+static gboolean move_window_to_workspace_cb(gpointer data)
+{
+	LayoutWindow *lw = data;
+	GdkWindow *window;
+
+	if (lw->options.workspace != -1)
+		{
+		window = gtk_widget_get_window(GTK_WIDGET(lw->window));
+		gdk_x11_window_move_to_desktop(window, lw->options.workspace);
+		}
+
+	return FALSE;
+}
+#endif
+#endif
+
 LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 				       const gchar *geometry)
 {
@@ -2438,6 +2586,8 @@ LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 	gtk_window_set_resizable(GTK_WINDOW(lw->window), TRUE);
 	gtk_container_set_border_width(GTK_CONTAINER(lw->window), 0);
 
+	g_signal_connect(G_OBJECT(lw->window), "button_release_event", G_CALLBACK(release_cb), lw);
+
 	if (options->save_window_positions)
 		{
 		hint_mask = GDK_HINT_USER_POS;
@@ -2462,6 +2612,11 @@ LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
 		gtk_window_move(GTK_WINDOW(lw->window), lw->options.main_window.x, lw->options.main_window.y);
 		if (lw->options.main_window.maximized) gtk_window_maximize(GTK_WINDOW(lw->window));
 //			}
+#ifdef GDK_WINDOWING_X11
+#if GTK_CHECK_VERSION(3,10,0)
+		g_idle_add(move_window_to_workspace_cb, lw);
+#endif
+#endif
 		}
 	else
 		{
@@ -2541,6 +2696,8 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_CHAR(*layout, order);
 	WRITE_NL(); WRITE_UINT(*layout, dir_view_type);
 	WRITE_NL(); WRITE_UINT(*layout, file_view_type);
+	WRITE_NL(); WRITE_UINT(*layout, dir_view_list_sort.method);
+	WRITE_NL(); WRITE_BOOL(*layout, dir_view_list_sort.ascend);
 	WRITE_NL(); WRITE_BOOL(*layout, show_marks);
 	WRITE_NL(); WRITE_BOOL(*layout, show_file_filter);
 	WRITE_NL(); WRITE_BOOL(*layout, show_thumbnails);
@@ -2548,7 +2705,6 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_CHAR(*layout, home_path);
 	WRITE_NL(); WRITE_CHAR(*layout, last_path);
 	WRITE_NL(); WRITE_UINT(*layout, startup_path);
-	WRITE_NL(); WRITE_BOOL(*layout, exit_on_close);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_INT(*layout, main_window.x);
@@ -2558,6 +2714,7 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_BOOL(*layout, main_window.maximized);
 	WRITE_NL(); WRITE_INT(*layout, main_window.hdivider_pos);
 	WRITE_NL(); WRITE_INT(*layout, main_window.vdivider_pos);
+	WRITE_NL(); WRITE_INT(*layout, workspace);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_INT(*layout, folder_window.vdivider_pos);
@@ -2580,6 +2737,7 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 
 	WRITE_NL(); WRITE_BOOL(*layout, toolbar_hidden);
 	WRITE_NL(); WRITE_BOOL(*layout, show_info_pixel);
+	WRITE_NL(); WRITE_BOOL(*layout, ignore_alpha);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_BOOL(*layout, bars_state.info);
@@ -2597,6 +2755,16 @@ void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent
 	WRITE_NL(); WRITE_INT(*layout, log_window.y);
 	WRITE_NL(); WRITE_INT(*layout, log_window.w);
 	WRITE_NL(); WRITE_INT(*layout, log_window.h);
+
+	WRITE_NL(); WRITE_INT(*layout, search_window.x);
+	WRITE_NL(); WRITE_INT(*layout, search_window.y);
+	WRITE_NL(); WRITE_INT(*layout, search_window.w);
+	WRITE_NL(); WRITE_INT(*layout, search_window.h);
+
+	WRITE_NL(); WRITE_INT(*layout, dupe_window.x);
+	WRITE_NL(); WRITE_INT(*layout, dupe_window.y);
+	WRITE_NL(); WRITE_INT(*layout, dupe_window.w);
+	WRITE_NL(); WRITE_INT(*layout, dupe_window.h);
 	WRITE_SEPARATOR();
 
 	WRITE_NL(); WRITE_BOOL(*layout, animate);
@@ -2618,6 +2786,7 @@ void layout_write_config(LayoutWindow *lw, GString *outstr, gint indent)
 
 	WRITE_SEPARATOR();
 	layout_toolbar_write_config(lw, TOOLBAR_MAIN, outstr, indent + 1);
+	layout_toolbar_write_config(lw, TOOLBAR_STATUS, outstr, indent + 1);
 
 	WRITE_NL(); WRITE_STRING("</layout>");
 }
@@ -2639,6 +2808,8 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 
 		if (READ_UINT(*layout, dir_view_type)) continue;
 		if (READ_UINT(*layout, file_view_type)) continue;
+		if (READ_UINT(*layout, dir_view_list_sort.method)) continue;
+		if (READ_BOOL(*layout, dir_view_list_sort.ascend)) continue;
 		if (READ_BOOL(*layout, show_marks)) continue;
 		if (READ_BOOL(*layout, show_file_filter)) continue;
 		if (READ_BOOL(*layout, show_thumbnails)) continue;
@@ -2646,7 +2817,6 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_CHAR(*layout, home_path)) continue;
 		if (READ_CHAR(*layout, last_path)) continue;
 		if (READ_UINT_CLAMP(*layout, startup_path, 0, STARTUP_PATH_HOME)) continue;
-		if (READ_BOOL(*layout, exit_on_close)) continue;
 
 		/* window positions */
 
@@ -2673,6 +2843,7 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_BOOL(*layout, tools_hidden)) continue;
 		if (READ_BOOL(*layout, toolbar_hidden)) continue;
 		if (READ_BOOL(*layout, show_info_pixel)) continue;
+		if (READ_BOOL(*layout, ignore_alpha)) continue;
 
 		if (READ_BOOL(*layout, bars_state.info)) continue;
 		if (READ_BOOL(*layout, bars_state.sort)) continue;
@@ -2689,7 +2860,18 @@ void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names
 		if (READ_INT(*layout, log_window.w)) continue;
 		if (READ_INT(*layout, log_window.h)) continue;
 
+		if (READ_INT(*layout, search_window.x)) continue;
+		if (READ_INT(*layout, search_window.y)) continue;
+		if (READ_INT(*layout, search_window.w)) continue;
+		if (READ_INT(*layout, search_window.h)) continue;
+
+		if (READ_INT(*layout, dupe_window.x)) continue;
+		if (READ_INT(*layout, dupe_window.y)) continue;
+		if (READ_INT(*layout, dupe_window.w)) continue;
+		if (READ_INT(*layout, dupe_window.h)) continue;
+
 		if (READ_BOOL(*layout, animate)) continue;
+		if (READ_INT(*layout, workspace)) continue;
 
 		log_printf("unknown attribute %s = %s\n", option, value);
 		}
@@ -2748,6 +2930,8 @@ static void layout_config_commandline(LayoutOptions *lop, gchar **path)
 		}
 }
 
+static gboolean first_found = FALSE;
+
 LayoutWindow *layout_new_from_config(const gchar **attribute_names, const gchar **attribute_values, gboolean use_commandline)
 {
 	LayoutOptions lop;
@@ -2758,8 +2942,12 @@ LayoutWindow *layout_new_from_config(const gchar **attribute_names, const gchar
 
 	if (attribute_names) layout_load_attributes(&lop, attribute_names, attribute_values);
 
-	if (use_commandline)
+	/* If multiple windows are specified in the config. file,
+	 * use the command line options only in the main window.
+	 */
+	if (use_commandline && !first_found)
 		{
+		first_found = TRUE;
 		layout_config_commandline(&lop, &path);
 		}
 	else
