@@ -395,11 +395,9 @@ static void animation_async_ready_cb(GObject *source_object, GAsyncResult *res,
 			{
 			if (!gdk_pixbuf_animation_is_static_image(animation->gpa))
 				{
-				if (animation->iter = gdk_pixbuf_animation_get_iter(animation->gpa,NULL))
+				animation->iter = gdk_pixbuf_animation_get_iter(animation->gpa, NULL);
+				if (animation->iter)
 					{
-					animation->iter = gdk_pixbuf_animation_get_iter(animation->gpa, NULL);
-					// FIXME: the reference count is being incremented here. Reason unknown.
-					g_object_unref(animation->gpa);
 					animation->data_adr = animation->lw->image->image_fd;
 					animation->delay = gdk_pixbuf_animation_iter_get_delay_time(animation->iter);
 					animation->valid = TRUE;
@@ -518,12 +516,14 @@ static void li_pop_menu_edit_cb(GtkWidget *widget, gpointer data)
 	file_util_start_editor_from_file(key, layout_image_get_fd(lw), lw->window);
 }
 
+#if !GTK_CHECK_VERSION(3,0,0)
 static void li_pop_menu_wallpaper_cb(GtkWidget *widget, gpointer data)
 {
 	LayoutWindow *lw = data;
 
 	layout_image_to_root(lw);
 }
+#endif
 
 static void li_pop_menu_alter_cb(GtkWidget *widget, gpointer data)
 {
@@ -863,6 +863,7 @@ static void layout_image_dnd_receive(GtkWidget *widget, GdkDragContext *context,
 {
 	LayoutWindow *lw = data;
 	gint i;
+	gchar *url;
 
 
 	for (i = 0; i < MAX_SPLIT_IMAGES; i++)
@@ -876,8 +877,13 @@ static void layout_image_dnd_receive(GtkWidget *widget, GdkDragContext *context,
 		layout_image_activate(lw, i, FALSE);
 		}
 
-
-	if (info == TARGET_URI_LIST || info == TARGET_APP_COLLECTION_MEMBER)
+	if (info == TARGET_TEXT_PLAIN)
+		{
+		url = g_strdup((gchar *)gtk_selection_data_get_data(selection_data));
+		download_web_file(url, FALSE, lw);
+		g_free(url);
+		}
+	else if (info == TARGET_URI_LIST || info == TARGET_APP_COLLECTION_MEMBER)
 		{
 		CollectionData *source;
 		GList *list;
@@ -1290,6 +1296,28 @@ gboolean layout_image_get_desaturate(LayoutWindow *lw)
 	return image_get_desaturate(lw->image);
 }
 
+void layout_image_set_overunderexposed(LayoutWindow *lw, gboolean overunderexposed)
+{
+	if (!layout_valid(&lw)) return;
+
+	image_set_overunderexposed(lw->image, overunderexposed);
+}
+
+gboolean layout_image_get_overunderexposed(LayoutWindow *lw)
+{
+	if (!layout_valid(&lw)) return FALSE;
+
+	return image_get_overunderexposed(lw->image);
+}
+
+void layout_image_set_ignore_alpha(LayoutWindow *lw, gboolean ignore_alpha)
+{
+   if (!layout_valid(&lw)) return;
+
+   lw->options.ignore_alpha = ignore_alpha;
+   image_set_ignore_alpha(lw->image, ignore_alpha);
+}
+
 /* stereo */
 /*
 gint layout_image_stereo_get(LayoutWindow *lw)
@@ -1382,6 +1410,11 @@ void layout_image_set_fd(LayoutWindow *lw, FileData *fd)
 	layout_image_slideshow_continue_check(lw);
 	layout_bars_new_image(lw);
 	layout_image_animate_new_file(lw);
+
+	if (fd)
+		{
+		image_chain_append_end(fd->path);
+		}
 }
 
 void layout_image_set_with_ahead(LayoutWindow *lw, FileData *fd, FileData *read_ahead_fd)
@@ -1781,7 +1814,6 @@ static void layout_image_button_cb(ImageWindow *imd, GdkEventButton *event, gpoi
 {
 	LayoutWindow *lw = data;
 	GtkWidget *menu;
-	FileData *dir_fd;
 
 	switch (event->button)
 		{
@@ -1805,16 +1837,6 @@ static void layout_image_button_cb(ImageWindow *imd, GdkEventButton *event, gpoi
 				}
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, event->time);
 			break;
-		case MOUSE_BUTTON_BACK:
-			dir_fd = file_data_new_dir(history_chain_back());
-			layout_set_fd(lw, dir_fd);
-			file_data_unref(dir_fd);
-			break;
-		case MOUSE_BUTTON_FORWARD:
-			dir_fd = file_data_new_dir(history_chain_forward());
-			layout_set_fd(lw, dir_fd);
-			file_data_unref(dir_fd);
-			break;
 		default:
 			break;
 		}
