@@ -61,6 +61,10 @@
 #include <clutter-gtk/clutter-gtk.h>
 #endif
 
+#ifdef HAVE_GTHREAD
+/* FIXME: see below */
+#include <X11/Xlib.h>
+#endif
 
 gboolean thumb_format_changed = FALSE;
 static RemoteConnection *remote_connection = NULL;
@@ -134,7 +138,7 @@ static void parse_command_line_add_file(const gchar *file_path, gchar **path, gc
 		{
 		if (!*path) *path = remove_level_from_path(path_parsed);
 		if (!*file) *file = g_strdup(path_parsed);
-		*list = g_list_prepend(*list, file_data_new_group(path_parsed));
+		*list = g_list_prepend(*list, file_data_new_no_grouping(path_parsed));
 		}
 }
 
@@ -224,6 +228,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 	gchar *app_lock;
 	gchar *pwd;
 	gchar *current_dir;
+	gchar *geometry = NULL;
 
 	command_line = g_new0(CommandLine, 1);
 
@@ -259,6 +264,9 @@ static void parse_command_line(gint argc, gchar *argv[])
 				parse_command_line_process_file(cmd_all, &command_line->path, &command_line->file,
 								&list, &command_line->collection_list, &first_dir);
 				}
+			else if (download_web_file(cmd_line, FALSE, NULL))
+				{
+				}
 			else if (is_collection(cmd_line))
 				{
 				gchar *path = NULL;
@@ -272,6 +280,10 @@ static void parse_command_line(gint argc, gchar *argv[])
 				{
 				/* do nothing but do not produce warnings */
 				}
+			else if (strncmp(cmd_line, "--disable-clutter", 17) == 0 && (cmd_line[17] == '\0'))
+				{
+				/* do nothing but do not produce warnings */
+				}
 			else if (strcmp(cmd_line, "+t") == 0 ||
 				 strcmp(cmd_line, "--with-tools") == 0)
 				{
@@ -335,6 +347,14 @@ static void parse_command_line(gint argc, gchar *argv[])
 				{
 				set_regexp(g_strdup(cmd_line+3));
 				}
+			else if (strncmp(cmd_line, "-n", 2) == 0)
+				{
+				command_line->new_instance = TRUE;
+				}
+			else if (strncmp(cmd_line, "--new-instance", 14) == 0)
+				{
+				command_line->new_instance = TRUE;
+				}
 			else if (strcmp(cmd_line, "-rh") == 0 ||
 				 strcmp(cmd_line, "--remote-help") == 0)
 				{
@@ -370,6 +390,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 				print_term(FALSE, _("  -l, --list [files] [collections] open collection window for command line\n"));
 				print_term(FALSE, _("      --blank                      start with blank file list\n"));
 				print_term(FALSE, _("      --geometry=XxY+XOFF+YOFF     set main window location\n"));
+				print_term(FALSE, _("  -n, --new-instance               open a new instance of Geeqie\n"));
 				print_term(FALSE, _("  -r, --remote                     send following commands to open window\n"));
 				print_term(FALSE, _("  -rh,--remote-help                print remote command list\n"));
 #ifdef DEBUG
@@ -379,7 +400,8 @@ static void parse_command_line(gint argc, gchar *argv[])
 				print_term(FALSE, _("  +w, --show-log-window            show log window\n"));
 				print_term(FALSE, _("  -o:<file>, --log-file:<file>     save log data to file\n"));
 				print_term(FALSE, _("  -v, --version                    print version info\n"));
-				print_term(FALSE, _("  -h, --help                       show this message\n\n"));
+				print_term(FALSE, _("  -h, --help                       show this message\n"));
+				print_term(FALSE, _("      --disable-clutter            disable use of Clutter library (i.e. GPU accel.)\n\n"));
 
 #if 0
 				/* these options are not officially supported!
@@ -415,16 +437,24 @@ static void parse_command_line(gint argc, gchar *argv[])
 		}
 	g_free(first_dir);
 
-	/* If Geeqie is already running, prevent a second instance
-	 * from being started. Open a new window instead.
-	 */
-	app_lock = g_build_filename(get_rc_dir(), ".command", NULL);
-	if (remote_server_exists(app_lock) && !remote_do)
+	if (!command_line->new_instance)
 		{
-		remote_do = TRUE;
-		remote_list = g_list_append(remote_list, "--new-window");
-	}
-	g_free(app_lock);
+		/* If Geeqie is already running, prevent a second instance
+		 * from being started. Open a new window instead.
+		 */
+		app_lock = g_build_filename(get_rc_dir(), ".command", NULL);
+		if (remote_server_exists(app_lock) && !remote_do)
+			{
+			remote_do = TRUE;
+			if (command_line->geometry)
+				{
+				geometry = g_strdup_printf("--geometry=%s", command_line->geometry);
+				remote_list = g_list_prepend(remote_list, geometry);
+				}
+			remote_list = g_list_prepend(remote_list, "--new-window");
+		}
+		g_free(app_lock);
+		}
 
 	if (remote_do)
 		{
@@ -455,6 +485,7 @@ static void parse_command_line(gint argc, gchar *argv[])
 		g_free(pwd);
 		g_free(current_dir);
 		}
+	g_free(geometry);
 	g_list_free(remote_list);
 
 	if (list && list->next)
@@ -514,6 +545,31 @@ static void parse_command_line_for_debug_option(gint argc, gchar *argv[])
 #endif
 }
 
+#ifdef HAVE_CLUTTER
+static gboolean parse_command_line_for_clutter_option(gint argc, gchar *argv[])
+{
+	const gchar *clutter_option = "--disable-clutter";
+	gint len = strlen(clutter_option);
+	gboolean ret = FALSE;
+
+	if (argc > 1)
+		{
+		gint i;
+
+		for (i = 1; i < argc; i++)
+			{
+			const gchar *cmd_line = argv[i];
+			if (strncmp(cmd_line, clutter_option, len) == 0)
+				{
+				ret = TRUE;
+				}
+			}
+		}
+
+	return ret;
+}
+#endif
+
 /*
  *-----------------------------------------------------------------------------
  * startup, init, and exit
@@ -698,6 +754,8 @@ static void gtkrc_load(void)
 static void exit_program_final(void)
 {
 	LayoutWindow *lw = NULL;
+	GList *list;
+	LayoutWindow *tmp_lw;
 
 	 /* make sure that external editors are loaded, we would save incomplete configuration otherwise */
 	layout_editors_reload_finish();
@@ -706,6 +764,21 @@ static void exit_program_final(void)
 
 	collect_manager_flush();
 
+	/* Save the named windows */
+	if (layout_window_list && layout_window_list->next)
+		{
+		list = layout_window_list;
+		while (list)
+			{
+			tmp_lw = list->data;
+			if (!g_str_has_prefix(tmp_lw->options.id, "lw"))
+				{
+				save_layout(list->data);
+				}
+			list = list->next;
+			}
+		}
+
 	save_options(options);
 	keys_save();
 	accel_map_save();
@@ -826,11 +899,16 @@ gint main(gint argc, gchar *argv[])
 	CollectionData *first_collection = NULL;
 	gchar *buf;
 	CollectionData *cd = NULL;
-	gchar *app_lock;
+	gboolean disable_clutter = FALSE;
 
 #ifdef HAVE_GTHREAD
 #if !GLIB_CHECK_VERSION(2,32,0)
 	g_thread_init(NULL);
+#endif
+#ifdef HAVE_CLUTTER
+/* FIXME: see below */
+	putenv("LIBGL_ALWAYS_INDIRECT=1");
+	XInitThreads();
 #endif
 	gdk_threads_init();
 	gdk_threads_enter();
@@ -873,10 +951,18 @@ gint main(gint argc, gchar *argv[])
 	parse_command_line_for_debug_option(argc, argv);
 	DEBUG_1("%s main: gtk_init", get_exec_time());
 #ifdef HAVE_CLUTTER
-	if (gtk_clutter_init(&argc, &argv) != CLUTTER_INIT_SUCCESS)
+	if (parse_command_line_for_clutter_option(argc, argv))
 		{
-		log_printf("Can't initialize clutter-gtk.\n");
-		exit(1);
+		disable_clutter	= TRUE;
+		gtk_init(&argc, &argv);
+		}
+	else
+		{
+		if (gtk_clutter_init(&argc, &argv) != CLUTTER_INIT_SUCCESS)
+			{
+			log_printf("Can't initialize clutter-gtk.\nStart Geeqie with the option \"geeqie --disable-clutter\"");
+			exit(1);
+			}
 		}
 #else
 	gtk_init(&argc, &argv);
@@ -898,6 +984,10 @@ gint main(gint argc, gchar *argv[])
 	DEBUG_1("%s main: setting default options before commandline handling", get_exec_time());
 	options = init_options(NULL);
 	setup_default_options(options);
+	if (disable_clutter)
+		{
+		options->disable_gpu = TRUE;
+		}
 
 	DEBUG_1("%s main: parse_command_line", get_exec_time());
 	parse_command_line(argc, argv);
@@ -908,6 +998,7 @@ gint main(gint argc, gchar *argv[])
 	mkdir_if_not_exists(get_collections_dir());
 	mkdir_if_not_exists(get_thumbnails_cache_dir());
 	mkdir_if_not_exists(get_metadata_cache_dir());
+	mkdir_if_not_exists(get_window_layouts_dir());
 
 	setup_env_path();
 
@@ -925,6 +1016,23 @@ gint main(gint argc, gchar *argv[])
 		filter_rebuild();
 		}
 
+#ifdef HAVE_CLUTTER
+/* FIXME: For the background of this see:
+ * https://github.com/BestImageViewer/geeqie/issues/397
+ * The feature CLUTTER_FEATURE_SWAP_EVENTS indictates if the
+ * system is liable to exhibit this problem.
+ * The user is provided with an override in Preferences/Behavior
+ */
+	if (!options->override_disable_gpu && !options->disable_gpu)
+		{
+		DEBUG_1("CLUTTER_FEATURE_SWAP_EVENTS %d",clutter_feature_available(CLUTTER_FEATURE_SWAP_EVENTS));
+		if (clutter_feature_available(CLUTTER_FEATURE_SWAP_EVENTS) != 0)
+			{
+			options->disable_gpu = TRUE;
+			}
+		}
+#endif
+
 	/* handle missing config file and commandline additions*/
 	if (!layout_window_list)
 		{
