@@ -25,6 +25,8 @@
 #include "cache_maint.h"
 #include "collect.h"
 #include "collect-dlg.h"
+#include "collect-io.h"
+#include "collect-table.h"
 #include "dupe.h"
 #include "editors.h"
 #include "filedata.h"
@@ -114,7 +116,7 @@ GtkWidget *submenu_add_edit(GtkWidget *menu, GtkWidget **menu_item, GCallback fu
 	GtkWidget *item;
 	GtkWidget *submenu;
 
-	item = menu_item_add(menu, _("_Edit"), NULL, NULL);
+	item = menu_item_add(menu, _("Plugins"), NULL, NULL);
 
 	submenu = gtk_menu_new();
 	g_object_set_data(G_OBJECT(submenu), "submenu_data", data);
@@ -147,7 +149,10 @@ gchar *sort_type_get_text(SortType method)
 			return _("Sort by file creation date");
 			break;
 		case SORT_EXIFTIME:
-			return _("Sort by Exif-date");
+			return _("Sort by Exif date original");
+			break;
+		case SORT_EXIFTIMEDIGITIZED:
+			return _("Sort by Exif date digitized");
 			break;
 		case SORT_NONE:
 			return _("Unsorted");
@@ -158,6 +163,12 @@ gchar *sort_type_get_text(SortType method)
 		case SORT_NUMBER:
 			return _("Sort by number");
 			break;
+		case SORT_RATING:
+			return _("Sort by rating");
+			break;
+		case SORT_CLASS:
+			return _("Sort by class");
+			break;
 		case SORT_NAME:
 		default:
 			return _("Sort by name");
@@ -204,7 +215,10 @@ GtkWidget *submenu_add_sort(GtkWidget *menu, GCallback func, gpointer data,
 	submenu_add_sort_item(submenu, func, SORT_TIME, show_current, type);
 	submenu_add_sort_item(submenu, func, SORT_CTIME, show_current, type);
 	submenu_add_sort_item(submenu, func, SORT_EXIFTIME, show_current, type);
+	submenu_add_sort_item(submenu, func, SORT_EXIFTIMEDIGITIZED, show_current, type);
 	submenu_add_sort_item(submenu, func, SORT_SIZE, show_current, type);
+	submenu_add_sort_item(submenu, func, SORT_RATING, show_current, type);
+	submenu_add_sort_item(submenu, func, SORT_CLASS, show_current, type);
 	if (include_path) submenu_add_sort_item(submenu, func, SORT_PATH, show_current, type);
 	if (include_none) submenu_add_sort_item(submenu, func, SORT_NONE, show_current, type);
 
@@ -220,6 +234,106 @@ GtkWidget *submenu_add_sort(GtkWidget *menu, GCallback func, gpointer data,
 	return submenu;
 }
 
+GtkWidget *submenu_add_dir_sort(GtkWidget *menu, GCallback func, gpointer data,
+			    gboolean include_none, gboolean include_path,
+			    gboolean show_current, SortType type)
+{
+	GtkWidget *submenu;
+
+	submenu = gtk_menu_new();
+	g_object_set_data(G_OBJECT(submenu), "submenu_data", data);
+
+	submenu_add_sort_item(submenu, func, SORT_NAME, show_current, type);
+	submenu_add_sort_item(submenu, func, SORT_TIME, show_current, type);
+	if (include_path) submenu_add_sort_item(submenu, func, SORT_PATH, show_current, type);
+	if (include_none) submenu_add_sort_item(submenu, func, SORT_NONE, show_current, type);
+
+	if (menu)
+		{
+		GtkWidget *item;
+
+		item = menu_item_add(menu, _("Sort"), NULL, NULL);
+		gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+		return item;
+		}
+
+	return submenu;
+}
+
+gchar *zoom_type_get_text(ZoomMode method)
+{
+	switch (method)
+		{
+		case ZOOM_RESET_ORIGINAL:
+			return _("Zoom to original size");
+			break;
+		case ZOOM_RESET_FIT_WINDOW:
+			return _("Fit image to window");
+			break;
+		case ZOOM_RESET_NONE:
+			return _("Leave Zoom at previous setting");
+			break;
+		default:
+			return _("Zoom to original size");
+			break;
+		}
+
+	return "";
+}
+
+static GtkWidget *submenu_add_zoom_item(GtkWidget *menu,
+					GCallback func, ZoomMode mode,
+					gboolean show_current, ZoomMode show_mode)
+{
+	GtkWidget *item;
+
+	if (show_current)
+		{
+		item = menu_item_add_radio(menu,
+					   zoom_type_get_text(mode), GINT_TO_POINTER((gint)mode), (mode == show_mode),
+					   func, GINT_TO_POINTER((gint)mode));
+		}
+	else
+		{
+		item = menu_item_add(menu, zoom_type_get_text(mode),
+				     func, GINT_TO_POINTER((gint)mode));
+		}
+
+	return item;
+}
+
+GtkWidget *submenu_add_zoom(GtkWidget *menu, GCallback func, gpointer data,
+			    gboolean include_none, gboolean include_path,
+			    gboolean show_current, ZoomMode mode)
+{
+	GtkWidget *submenu;
+
+	if (!menu)
+		{
+		submenu = gtk_menu_new();
+		g_object_set_data(G_OBJECT(submenu), "submenu_data", data);
+		}
+	else
+		{
+		submenu = menu;
+		}
+
+	submenu_add_zoom_item(submenu, func, ZOOM_RESET_ORIGINAL, show_current, mode);
+	submenu_add_zoom_item(submenu, func, ZOOM_RESET_FIT_WINDOW, show_current, mode);
+	submenu_add_zoom_item(submenu, func, ZOOM_RESET_NONE, show_current, mode);
+
+	if (menu)
+		{
+		GtkWidget *item;
+
+		item = menu_item_add(menu, _("Zoom"), NULL, NULL);
+		gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+		return item;
+		}
+
+	return submenu;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * altering
@@ -231,13 +345,13 @@ gchar *alter_type_get_text(AlterType type)
 	switch (type)
 		{
 		case ALTER_ROTATE_90:
-			return _("_Rotate clockwise");
+			return _("_Rotate clockwise 90°");
 			break;
 		case ALTER_ROTATE_90_CC:
-			return _("Rotate _counterclockwise");
+			return _("Rotate _counterclockwise 90°");
 			break;
 		case ALTER_ROTATE_180:
-			return _("Rotate _180");
+			return _("Rotate _180°");
 			break;
 		case ALTER_MIRROR:
 			return _("_Mirror");
@@ -301,4 +415,97 @@ GtkWidget *submenu_add_alter(GtkWidget *menu, GCallback func, gpointer data)
 {
 	return real_submenu_add_alter(menu, func, data, NULL);
 }
+
+/*
+ *-----------------------------------------------------------------------------
+ * collections
+ *-----------------------------------------------------------------------------
+ */
+
+/**
+ * @brief Add submenu consisting of "New collection", and list of existing collections to a right-click menu.
+ * @param[in] menu 
+ * @param[in] func 
+ * @param[in] collection_list Type gchar
+ * @param[in] data 
+ * 
+ *  Used by all image windows
+ */
+static void add_collection_list(GtkWidget *menu, GCallback func,
+								GList *collection_list, gpointer data)
+{
+	GList *work;
+	gint index = 0; /* index to existing collection list menu item selected */
+
+	work = collection_list;
+	while (work)
+		{
+		const gchar *collection_name = work->data;
+
+		menu_item_add(menu, collection_name, func,
+													GINT_TO_POINTER(index));
+		work = work->next;
+		index++;
+		}
+}
+
+GtkWidget *submenu_add_collections(GtkWidget *menu, GtkWidget **menu_item,
+										GCallback func, gpointer data)
+{
+	GtkWidget *item;
+	GtkWidget *submenu;
+	GList *collection_list = NULL;
+
+	item = menu_item_add(menu, _("_Add to Collection"), NULL, NULL);
+
+	submenu = gtk_menu_new();
+	g_object_set_data(G_OBJECT(submenu), "submenu_data", data);
+
+	menu_item_add_stock_sensitive(submenu, _("New collection"),
+					GTK_STOCK_INDEX, TRUE, G_CALLBACK(func), GINT_TO_POINTER(-1));
+	menu_item_add_divider(submenu);
+
+	collect_manager_list(&collection_list,NULL,NULL);
+	add_collection_list(submenu, func, collection_list, data);
+
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+	if (menu_item) *menu_item = item;
+
+	g_list_free(collection_list);
+
+	return submenu;
+}
+
+/**
+ * @brief Add file selection list to a collection
+ * @param[in] selection_list Selection list of ::_FileData
+ * @param[in] data Index to the collection list menu item selected, or -1 for new collection
+ * 
+ * 
+ */
+void pop_menu_collections(GList *selection_list, gpointer data)
+{
+	CollectWindow *cw;
+	gchar *collection_name;
+	GList *collection_list = NULL;
+	gchar *name;
+	const gint index = GPOINTER_TO_INT(data);
+
+	if (index >= 0)
+		{
+		collect_manager_list(&collection_list, NULL, NULL);
+		collection_name = g_list_nth_data(collection_list, index);
+		name = collection_path(collection_name);
+		cw = collection_window_new(name);
+		g_free(name);
+		string_list_free(collection_list);
+		}
+	else
+		{
+		cw = collection_window_new(NULL);
+		}
+
+	collection_table_add_filelist(cw->table, selection_list);
+}
+
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
