@@ -253,12 +253,12 @@ int runcmd(gchar *cmd)
  */
 gint date_get_first_day_of_week()
 {
-	gchar *dot;
-	gchar *current_locale;
-
 #ifdef HAVE__NL_TIME_FIRST_WEEKDAY
 	return nl_langinfo(_NL_TIME_FIRST_WEEKDAY)[0];
 #else
+	gchar *dot;
+	gchar *current_locale;
+
 	current_locale = setlocale(LC_ALL, NULL);
 	dot = strstr(current_locale, ".");
 	if ((strncmp(dot - 2, "US", 2) == 0) || (strncmp(dot - 2, "MX", 2) == 0) || (strncmp(dot - 2, "CA", 2) == 0))
@@ -376,4 +376,36 @@ gchar *get_symbolic_link(const gchar *path_utf8)
 	return ret;
 }
 
+gint get_cpu_cores(void)
+{
+	FILE *cpuinfo = fopen("/proc/cpuinfo", "rb");
+	char *arg = 0;
+	size_t size = 0;
+	int cores = 1;
+	gchar *siblings_line;
+	gchar *siblings_str;
+
+	while(getline(&arg, &size, cpuinfo) != -1)
+		{
+		siblings_line = g_strrstr(arg, "siblings");
+		if (siblings_line)
+			{
+			siblings_str = g_strrstr(siblings_line, ":");
+			if (siblings_str)
+				{
+				cores = g_ascii_strtoll(siblings_str + 1, NULL, 0);
+				}
+			}
+		}
+	free(arg);
+	fclose(cpuinfo);
+
+	return cores;
+}
+
+void tree_path_free_wrapper(void *data, void *useradata)
+{
+	gtk_tree_path_free(data);
+}
+
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
