@@ -32,6 +32,9 @@
 #include "ui_fileops.h"
 #include "window.h"
 
+ConfOptions *options;
+CommandLine *command_line;
+
 ConfOptions *init_options(ConfOptions *options)
 {
 	gint i;
@@ -48,6 +51,7 @@ ConfOptions *init_options(ConfOptions *options)
 	options->color_profile.render_intent = 0;
 
 	options->dnd_icon_size = 48;
+	options->dnd_default_action = DND_ACTION_ASK;
 	options->duplicates_similarity_threshold = 99;
 	options->rot_invariant_sim = TRUE;
 	options->sort_totals = FALSE;
@@ -87,6 +91,7 @@ ConfOptions *init_options(ConfOptions *options)
 	options->marks_save = TRUE;
 	options->with_rename = FALSE;
 	options->collections_on_top = FALSE;
+	options->hide_window_in_fullscreen = TRUE;
 
 	memset(&options->image.border_color, 0, sizeof(options->image.border_color));
 	memset(&options->image.alpha_color_1, 0, sizeof(options->image.alpha_color_1));
@@ -160,6 +165,7 @@ ConfOptions *init_options(ConfOptions *options)
 	options->show_icon_names = TRUE;
 	options->show_star_rating = FALSE;
 	options->show_predefined_keyword_tree = TRUE;
+	options->expand_menu_toolbar = FALSE;
 
 	options->slideshow.delay = 50;
 	options->slideshow.random = FALSE;
@@ -207,6 +213,16 @@ ConfOptions *init_options(ConfOptions *options)
 
 	options->disabled_plugins = NULL;
 
+	options->mouse_button_8 = g_strdup("Back");
+	options->mouse_button_9 = g_strdup("Forward");
+
+	options->disable_gpu = FALSE;
+	options->override_disable_gpu = FALSE;
+
+	for (i = 0; i < FILE_FORMAT_CLASSES; i++)
+		{
+		options->class_filter[i] = TRUE;
+		}
 	return options;
 }
 
@@ -269,6 +285,8 @@ LayoutOptions *init_layout_options(LayoutOptions *options)
 	memset(options, 0, sizeof(LayoutOptions));
 
 	options->dir_view_type = DIRVIEW_LIST;
+	options->dir_view_list_sort.ascend = TRUE;
+	options->dir_view_list_sort.method = SORT_NAME;
 	options->file_view_type = FILEVIEW_LIST;
 	options->float_window.h = 450;
 	options->float_window.vdivider_pos = -1;
@@ -283,6 +301,14 @@ LayoutOptions *init_layout_options(LayoutOptions *options)
 	options->main_window.w = 720;
 	options->main_window.x = 0;
 	options->main_window.y = 0;
+	options->dupe_window.w = 800;
+	options->dupe_window.h = 400;
+	options->dupe_window.x = 100;
+	options->dupe_window.y = 100;
+	options->search_window.w = 700;
+	options->search_window.h = 650;
+	options->search_window.x = 100;
+	options->search_window.y = 100;
 	options->folder_window.vdivider_pos = 100;
 	options->order = g_strdup("123");
 	options->show_directory_date = FALSE;
@@ -301,6 +327,8 @@ LayoutOptions *init_layout_options(LayoutOptions *options)
 	options->bars_state.hidden = FALSE;
 	options->log_window.w = 520;
 	options->log_window.h = 400;
+	options->split_pane_sync = FALSE;
+	options->workspace = -1;
 	return options;
 }
 
@@ -328,7 +356,7 @@ void save_options(ConfOptions *options)
 	sync_options_with_current_state(options);
 
 	rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
-	save_config_to_file(rc_path, options);
+	save_config_to_file(rc_path, options, NULL);
 	g_free(rc_path);
 }
 
