@@ -38,6 +38,8 @@ struct _ConfOptions
 	gboolean show_guidelines;
 	gboolean draw_rectangle;
 	gboolean show_predefined_keyword_tree;
+	gboolean overunderexposed;
+	gboolean expand_menu_toolbar;
 
 	/* various */
 	gboolean tree_descend_subdirs;
@@ -55,6 +57,7 @@ struct _ConfOptions
 
 	gint open_recent_list_maxsize;
 	gint dnd_icon_size;
+	DnDAction dnd_default_action;
 	gint clipboard_selection;
 
 	gboolean save_window_positions;
@@ -70,6 +73,7 @@ struct _ConfOptions
 
 	gboolean with_rename;
 	gboolean collections_on_top;
+	gboolean hide_window_in_fullscreen;
 
 	gchar *help_search_engine;
 
@@ -316,13 +320,24 @@ struct _ConfOptions
 		gchar *template_string;
 	} printer;
 
+	/* user-definable mouse buttons */
+	gchar *mouse_button_8;
+	gchar *mouse_button_9;
+
+	/* class file filter */
+	gboolean class_filter[FILE_FORMAT_CLASSES];
+
 	gboolean read_metadata_in_idle;
 
+	/* GPU - see main.c */
+	gboolean disable_gpu;
+	gboolean override_disable_gpu;
+
 	GList *disabled_plugins;
 };
 
-ConfOptions *options;
-CommandLine *command_line;
+extern ConfOptions *options;
+extern CommandLine *command_line;
 
 ConfOptions *init_options(ConfOptions *options);
 void setup_default_options(ConfOptions *options);
