@@ -334,6 +334,7 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_CHAR(*options, image_l_click_video_editor);
 	WRITE_NL(); WRITE_INT(*options, open_recent_list_maxsize);
 	WRITE_NL(); WRITE_INT(*options, dnd_icon_size);
+	WRITE_NL(); WRITE_UINT(*options, dnd_default_action);
 	WRITE_NL(); WRITE_BOOL(*options, place_dialogs_under_mouse);
 	WRITE_NL(); WRITE_INT(*options, clipboard_selection);
 
@@ -342,6 +343,7 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_BOOL(*options, tools_restore_state);
 	WRITE_NL(); WRITE_BOOL(*options, save_dialog_window_positions);
 	WRITE_NL(); WRITE_BOOL(*options, show_window_ids);
+	WRITE_NL(); WRITE_BOOL(*options, expand_menu_toolbar);
 
 	WRITE_NL(); WRITE_UINT(*options, log_window_lines);
 	WRITE_NL(); WRITE_BOOL(*options, log_window.timer_data);
@@ -351,6 +353,7 @@ static void write_global_attributes(GString *outstr, gint indent)
 
 	WRITE_NL(); WRITE_BOOL(*options, with_rename);
 	WRITE_NL(); WRITE_BOOL(*options, collections_on_top);
+	WRITE_NL(); WRITE_BOOL(*options, hide_window_in_fullscreen);
 
 	/* File operations Options */
 	WRITE_NL(); WRITE_BOOL(*options, file_ops.enable_in_place_rename);
@@ -510,6 +513,15 @@ static void write_global_attributes(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_BOOL(*options, printer.show_image_text);
 	WRITE_NL(); WRITE_BOOL(*options, printer.show_page_text);
 	WRITE_SEPARATOR();
+
+	/* user-definable mouse buttons */
+	WRITE_NL(); WRITE_CHAR(*options, mouse_button_8);
+	WRITE_NL(); WRITE_CHAR(*options, mouse_button_9);
+	WRITE_SEPARATOR();
+
+	/* GPU - see main.c */
+	WRITE_NL(); WRITE_BOOL(*options, override_disable_gpu);
+	WRITE_SEPARATOR();
 }
 
 static void write_color_profile(GString *outstr, gint indent)
@@ -557,6 +569,23 @@ static void write_marks_tooltips(GString *outstr, gint indent)
 	WRITE_NL(); WRITE_STRING("</marks_tooltips>");
 }
 
+static void write_class_filter(GString *outstr, gint indent)
+{
+	gint i;
+
+	WRITE_NL(); WRITE_STRING("<class_filter>");
+	indent++;
+	for (i = 0; i < FILE_FORMAT_CLASSES; i++)
+		{
+		WRITE_NL(); WRITE_STRING("<filter_type ");
+		write_char_option(outstr, indent, "filter", format_class_list[i]);
+		write_bool_option(outstr, indent, "enabled", options->class_filter[i]);
+		WRITE_STRING("/>");
+		}
+	indent--;
+	WRITE_NL(); WRITE_STRING("</class_filter>");
+}
+
 static void write_disabled_plugins(GString *outstr, gint indent)
 {
 	GtkTreeIter iter;
@@ -596,7 +625,7 @@ static void write_disabled_plugins(GString *outstr, gint indent)
  *-----------------------------------------------------------------------------
  */
 
-gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options)
+gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options, LayoutWindow *lw)
 {
 	SecureSaveInfo *ssi;
 	gchar *rc_pathl;
@@ -629,39 +658,51 @@ gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options)
 	WRITE_STRING("<gq>\n");
 	indent++;
 
-	WRITE_NL(); WRITE_STRING("<global\n");
-	indent++;
-	write_global_attributes(outstr, indent + 1);
-	indent--;
-	WRITE_STRING(">\n");
+	if (!lw)
+		{
+		WRITE_NL(); WRITE_STRING("<global\n");
+		indent++;
+		write_global_attributes(outstr, indent + 1);
+		indent--;
+		WRITE_STRING(">\n");
 
-	indent++;
+		indent++;
 
-	write_color_profile(outstr, indent);
+		write_color_profile(outstr, indent);
 
-	WRITE_SEPARATOR();
-	filter_write_list(outstr, indent);
+		WRITE_SEPARATOR();
+		filter_write_list(outstr, indent);
 
-	WRITE_SEPARATOR();
-	write_marks_tooltips(outstr, indent);
+		WRITE_SEPARATOR();
+		write_marks_tooltips(outstr, indent);
 
-	WRITE_SEPARATOR();
-	write_disabled_plugins(outstr, indent);
+		WRITE_SEPARATOR();
+		write_disabled_plugins(outstr, indent);
 
-	WRITE_SEPARATOR();
-	keyword_tree_write_config(outstr, indent);
-	indent--;
-	WRITE_NL(); WRITE_STRING("</global>\n");
+		WRITE_SEPARATOR();
+		write_class_filter(outstr, indent);
 
+		WRITE_SEPARATOR();
+		keyword_tree_write_config(outstr, indent);
+		indent--;
+		WRITE_NL(); WRITE_STRING("</global>\n");
+		}
 	WRITE_SEPARATOR();
 
 	/* Layout Options */
-	work = layout_window_list;
-	while (work)
+	if (!lw)
+		{
+		work = layout_window_list;
+		while (work)
+			{
+			LayoutWindow *lw = work->data;
+			layout_write_config(lw, outstr, indent);
+			work = work->next;
+			}
+		}
+	else
 		{
-		LayoutWindow *lw = work->data;
 		layout_write_config(lw, outstr, indent);
-		work = work->next;
 		}
 
 	indent--;
@@ -707,7 +748,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, update_on_time_change)) continue;
 
 		if (READ_UINT_CLAMP(*options, duplicates_similarity_threshold, 0, 100)) continue;
-		if (READ_UINT_CLAMP(*options, duplicates_match, 0, DUPE_MATCH_NAME_CI)) continue;
+		if (READ_UINT_CLAMP(*options, duplicates_match, 0, DUPE_MATCH_ALL)) continue;
 		if (READ_UINT_CLAMP(*options, duplicates_select_type, 0, DUPE_SELECT_GROUP2)) continue;
 		if (READ_BOOL(*options, duplicates_thumbnails)) continue;
 		if (READ_BOOL(*options, rot_invariant_sim)) continue;
@@ -723,6 +764,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 
 		if (READ_INT(*options, open_recent_list_maxsize)) continue;
 		if (READ_INT(*options, dnd_icon_size)) continue;
+		if (READ_UINT(*options, dnd_default_action)) continue;
 		if (READ_BOOL(*options, place_dialogs_under_mouse)) continue;
 		if (READ_INT(*options, clipboard_selection)) continue;
 
@@ -731,6 +773,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, tools_restore_state)) continue;
 		if (READ_BOOL(*options, save_dialog_window_positions)) continue;
 		if (READ_BOOL(*options, show_window_ids)) continue;
+		if (READ_BOOL(*options, expand_menu_toolbar)) continue;
 
 		if (READ_INT(*options, log_window_lines)) continue;
 		if (READ_BOOL(*options, log_window.timer_data)) continue;
@@ -739,6 +782,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_CHAR(*options, help_search_engine)) continue;
 
 		if (READ_BOOL(*options, collections_on_top)) continue;
+		if (READ_BOOL(*options, hide_window_in_fullscreen)) continue;
 
 		/* Properties dialog options */
 		if (READ_CHAR(*options, properties.tabs_order)) continue;
@@ -758,7 +802,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_UINT_CLAMP(*options, image.scroll_reset_method, 0, PR_SCROLL_RESET_COUNT - 1)) continue;
 		if (READ_INT(*options, image.tile_cache_max)) continue;
 		if (READ_INT(*options, image.image_cache_max)) continue;
-		if (READ_UINT_CLAMP(*options, image.zoom_quality, GDK_INTERP_NEAREST, GDK_INTERP_HYPER)) continue;
+		if (READ_UINT_CLAMP(*options, image.zoom_quality, GDK_INTERP_NEAREST, GDK_INTERP_BILINEAR)) continue;
 		if (READ_INT(*options, image.zoom_increment)) continue;
 		if (READ_BOOL(*options, image.enable_read_ahead)) continue;
 		if (READ_BOOL(*options, image.exif_rotate_enable)) continue;
@@ -777,7 +821,7 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, thumbnails.cache_into_dirs)) continue;
 		if (READ_BOOL(*options, thumbnails.use_xvpics)) continue;
 		if (READ_BOOL(*options, thumbnails.spec_standard)) continue;
-		if (READ_UINT_CLAMP(*options, thumbnails.quality, GDK_INTERP_NEAREST, GDK_INTERP_HYPER)) continue;
+		if (READ_UINT_CLAMP(*options, thumbnails.quality, GDK_INTERP_NEAREST, GDK_INTERP_BILINEAR)) continue;
 		if (READ_BOOL(*options, thumbnails.use_exif)) continue;
 		if (READ_INT(*options, thumbnails.collection_preview)) continue;
 		if (READ_BOOL(*options, thumbnails.use_ft_metadata)) continue;
@@ -893,6 +937,13 @@ static gboolean load_global_params(const gchar **attribute_names, const gchar **
 		if (READ_BOOL(*options, printer.show_image_text)) continue;
 		if (READ_BOOL(*options, printer.show_page_text)) continue;
 
+		/* user-definable mouse buttons */
+		if (READ_CHAR(*options, mouse_button_8)) continue;
+		if (READ_CHAR(*options, mouse_button_9)) continue;
+
+		/* GPU - see main.c */
+		if (READ_BOOL(*options, override_disable_gpu)) continue;
+
 		/* Dummy options */
 		if (READ_DUMMY(*options, image.dither_quality, "deprecated since 2012-08-13")) continue;
 
@@ -1041,6 +1092,49 @@ static void options_parse_marks_tooltips(GQParserData *parser_data, GMarkupParse
 		}
 }
 
+static void class_filter_load_filter_type(const gchar **attribute_names, const gchar **attribute_values)
+{
+	gint i;
+	gint index;
+
+	while (*attribute_names)
+		{
+		const gchar *option = *attribute_names++;
+		const gchar *value = *attribute_values++;
+		if (g_strcmp0("filter", option) == 0)
+			{
+			for (i = 0; i < FILE_FORMAT_CLASSES; i++)
+				{
+				if (g_strcmp0(format_class_list[i], value) == 0)
+					{
+					index = i;
+					}
+				}
+			continue;
+			}
+
+		if (READ_BOOL_FULL("enabled", options->class_filter[index]))
+			{
+			continue;
+			}
+		log_printf("unknown attribute %s = %s\n", option, value);
+		}
+}
+
+static void options_parse_class_filter(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+{
+	if (g_ascii_strcasecmp(element_name, "filter_type") == 0)
+		{
+		class_filter_load_filter_type(attribute_names, attribute_values);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+	else
+		{
+		log_printf("unexpected in <profile>: <%s>\n", element_name);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+}
+
 static void options_parse_disabled_plugins(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
 {
 	if (g_ascii_strcasecmp(element_name, "plugin") == 0)
@@ -1130,6 +1224,10 @@ static void options_parse_global(GQParserData *parser_data, GMarkupParseContext
 		options_load_marks_tooltips(parser_data, context, element_name, attribute_names, attribute_values, data, error);
 		options_parse_func_push(parser_data, options_parse_marks_tooltips, NULL, NULL);
 		}
+	else if (g_ascii_strcasecmp(element_name, "class_filter") == 0)
+		{
+		options_parse_func_push(parser_data, options_parse_class_filter, NULL, NULL);
+		}
 	else if (g_ascii_strcasecmp(element_name, "keyword_tree") == 0)
 		{
 		if (!keyword_tree) keyword_tree_new();
@@ -1210,17 +1308,21 @@ static void options_parse_bar(GQParserData *parser_data, GMarkupParseContext *co
 #ifdef HAVE_LIBCHAMPLAIN_GTK
 	else if (g_ascii_strcasecmp(element_name, "pane_gps") == 0)
 		{
-		GtkWidget *pane = bar_find_pane_by_id(bar, PANE_GPS, options_get_id(attribute_names, attribute_values));
-		if (pane)
+		/* Use this flag to determine if --disable-clutter has been issued */
+		if (!options->disable_gpu)
 			{
-			bar_pane_gps_update_from_config(pane, attribute_names, attribute_values);
+			GtkWidget *pane = bar_find_pane_by_id(bar, PANE_GPS, options_get_id(attribute_names, attribute_values));
+			if (pane)
+				{
+				bar_pane_gps_update_from_config(pane, attribute_names, attribute_values);
+				}
+			else
+				{
+				pane = bar_pane_gps_new_from_config(attribute_names, attribute_values);
+				bar_add(bar, pane);
+				}
+			options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 			}
-		else
-			{
-			pane = bar_pane_gps_new_from_config(attribute_names, attribute_values);
-			bar_add(bar, pane);
-			}
-		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 		}
 #endif
 #endif
@@ -1297,6 +1399,27 @@ static void options_parse_toolbar(GQParserData *parser_data, GMarkupParseContext
 		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
 		}
 }
+
+static void options_parse_statusbar(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
+{
+	LayoutWindow *lw = data;
+	if (g_ascii_strcasecmp(element_name, "toolitem") == 0)
+		{
+		layout_toolbar_add_from_config(lw, TOOLBAR_STATUS, attribute_names, attribute_values);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+	else if (g_ascii_strcasecmp(element_name, "clear") == 0)
+		{
+		layout_toolbar_clear(lw, TOOLBAR_STATUS);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+	else
+		{
+		log_printf("unexpected in <statusbar>: <%s>\n", element_name);
+		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		}
+}
+
 static void options_parse_dialogs(GQParserData *parser_data, GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, gpointer data, GError **error)
 {
 	if (g_ascii_strcasecmp(element_name, "window") == 0)
@@ -1339,7 +1462,7 @@ static void options_parse_layout(GQParserData *parser_data, GMarkupParseContext
 		}
 	else if (g_ascii_strcasecmp(element_name, "statusbar") == 0)
 		{
-		options_parse_func_push(parser_data, options_parse_leaf, NULL, NULL);
+		options_parse_func_push(parser_data, options_parse_statusbar, NULL, lw);
 		}
 	else if (g_ascii_strcasecmp(element_name, "dialogs") == 0)
 		{
