@@ -83,7 +83,7 @@ void options_parse_func_pop(GQParserData *parser_data);
 void options_parse_func_set_data(GQParserData *parser_data, gpointer data);
 
 
-gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options);
+gboolean save_config_to_file(const gchar *utf8_path, ConfOptions *options, LayoutWindow *lw);
 
 gboolean load_config_from_buf(const gchar *buf, gsize size, gboolean startup);
 gboolean load_config_from_file(const gchar *utf8_path, gboolean startup);
