@@ -118,13 +118,6 @@ static gchar *file_util_safe_dest(const gchar *path)
 	return dest;
 }
 
-static void file_util_safe_del_close_cb(GtkWidget *dialog, gpointer data)
-{
-	GenericDialog **gd = data;
-
-	*gd = NULL;
-}
-
 gboolean file_util_safe_unlink(const gchar *path)
 {
 	static GenericDialog *gd = NULL;
@@ -179,7 +172,6 @@ gboolean file_util_safe_unlink(const gchar *path)
 
 		if (result && !gd)
 			{
-			GtkWidget *button;
 			gchar *buf;
 
 			buf = g_strdup_printf(_("Unable to access or create the trash folder.\n\"%s\""), options->file_ops.safe_delete_path);
