@@ -39,8 +39,8 @@ typedef enum {
 	MOUSE_BUTTON_RIGHT	= 3,
 	MOUSE_BUTTON_WHEEL_UP	= 4,
 	MOUSE_BUTTON_WHEEL_DOWN	= 5,
-	MOUSE_BUTTON_BACK	= 8,
-	MOUSE_BUTTON_FORWARD	= 9
+	MOUSE_BUTTON_8	= 8,
+	MOUSE_BUTTON_9	= 9
 } MouseButton;
 
 typedef enum {
@@ -79,6 +79,13 @@ typedef enum {
 	SORT_CLASS
 } SortType;
 
+/* drag and drop default action */
+typedef enum {
+	DND_ACTION_ASK,
+	DND_ACTION_COPY,
+	DND_ACTION_MOVE
+} DnDAction;
+
 typedef enum {
 	ALTER_NONE,		/* do nothing */
 	ALTER_ROTATE_90,
@@ -144,10 +151,11 @@ typedef enum {
 	FORMAT_CLASS_META,
 	FORMAT_CLASS_VIDEO,
 	FORMAT_CLASS_COLLECTION,
-	FORMAT_CLASS_PDF,
+	FORMAT_CLASS_DOCUMENT,
 	FILE_FORMAT_CLASSES
 } FileFormatClass;
 
+/* defined in preferences.c */
 extern gchar *format_class_list[];
 
 typedef enum {
@@ -264,6 +272,25 @@ typedef enum {
 	STEREO_PIXBUF_NONE     = 3
 } StereoPixbufData;
 
+typedef enum {
+	BAR_SORT_MODE_FOLDER = 0,
+	BAR_SORT_MODE_COLLECTION,
+	BAR_SORT_MODE_COUNT
+} SortModeType;
+
+typedef enum {
+	BAR_SORT_COPY = 0,
+	BAR_SORT_MOVE,
+	BAR_SORT_FILTER,
+	BAR_SORT_ACTION_COUNT
+} SortActionType;
+
+typedef enum {
+	BAR_SORT_SELECTION_IMAGE = 0,
+	BAR_SORT_SELECTION_SELECTED,
+	BAR_SORT_SELECTION_COUNT
+} SortSelectionType;
+
 #define MAX_SPLIT_IMAGES 4
 
 typedef enum {
@@ -542,6 +569,7 @@ struct _ImageWindow
 	gboolean delay_flip;
 	gint orientation;
 	gboolean desaturate;
+	gboolean overunderexposed;
 	gint user_stereo;
 
 	gboolean mouse_wheel_mode;
@@ -609,6 +637,9 @@ struct _FileData {
 	gchar *sym_link;
 
 	SelectionType selected;  // Used by view_file_icon.
+
+	gint page_num;
+	gint page_total;
 };
 
 struct _LayoutOptions
@@ -621,11 +652,18 @@ struct _LayoutOptions
 	DirViewType dir_view_type;
 	FileViewType file_view_type;
 
+	struct {
+		SortType method;
+		gboolean ascend;
+	} dir_view_list_sort;
+
 	gboolean show_thumbnails;
 	gboolean show_marks;
 	gboolean show_file_filter;
 	gboolean show_directory_date;
 	gboolean show_info_pixel;
+	gboolean split_pane_sync;
+	gboolean ignore_alpha;
 
 	struct {
 		gint w;
@@ -667,6 +705,20 @@ struct _LayoutOptions
 		gint y;
 	} log_window;
 
+	struct {
+		gint w;
+		gint h;
+		gint x;
+		gint y;
+	} search_window;
+
+	struct {
+		gint w;
+		gint h;
+		gint x;
+		gint y;
+	} dupe_window;
+
 	gboolean tools_float;
 	gboolean tools_hidden;
 	gboolean toolbar_hidden;
@@ -684,9 +736,13 @@ struct _LayoutOptions
 
 	StartUpPath startup_path;
 
-	gboolean exit_on_close;
-
 	gboolean animate;
+	gint workspace;
+
+	SortActionType action;
+	SortModeType mode;
+	SortSelectionType selection;
+	gchar *filter_key;
 };
 
 struct _LayoutWindow
@@ -807,6 +863,7 @@ struct _LayoutWindow
 //	gint bar_width;
 
 	GtkWidget *exif_window;
+	GtkWidget *sar_window; /* Search and Run window */
 
 	AnimationData *animation;
 
@@ -872,6 +929,7 @@ struct _ViewFile
 		GtkWidget *frame;
 		gint count;
 		gint last_selected;
+		gboolean case_sensitive;
 	} file_filter;
 
 	FileData *dir_fd;
@@ -901,6 +959,10 @@ struct _ViewFile
 	gint active_mark;
 	gint clicked_mark;
 
+	/* stars */
+	FileData *stars_filedata;
+	guint stars_id;
+
 	/* refresh */
 	guint refresh_idle_id; /* event source id */
 	time_t time_refresh_set; /* time when refresh_idle_id was set */
@@ -1029,6 +1091,7 @@ struct _CommandLine
 	gchar *regexp;
 	gchar *log_file;
 	SecureSaveInfo *ssi;
+	gboolean new_instance;
 };
 
 #endif
