@@ -42,6 +42,8 @@
 #include "md5-util.h"
 
 #include "filefilter.h"
+#include "layout.h"
+#include "utilops.h"
 #include "secure_save.h"
 
 /*
@@ -300,6 +302,24 @@ const gchar *get_trash_dir(void)
 	return trash_dir;
 }
 
+const gchar *get_window_layouts_dir(void)
+{
+	static gchar *window_layouts_dir = NULL;
+
+	if (window_layouts_dir) return window_layouts_dir;
+
+	if (USE_XDG)
+		{
+		window_layouts_dir = g_build_filename(xdg_config_home_get(), GQ_APPNAME_LC, GQ_WINDOW_LAYOUTS_DIR, NULL);
+		}
+	else
+		{
+		window_layouts_dir = g_build_filename(get_rc_dir(), GQ_WINDOW_LAYOUTS_DIR, NULL);
+		}
+
+	return window_layouts_dir;
+}
+
 gboolean stat_utf8(const gchar *s, struct stat *st)
 {
 	gchar *sl;
@@ -713,9 +733,14 @@ gchar *get_current_dir(void)
 	return path8;
 }
 
+void list_free_wrapper(void *data, void *userdata)
+{
+	g_free(data);
+}
+
 void string_list_free(GList *list)
 {
-	g_list_foreach(list, (GFunc)g_free, NULL);
+	g_list_foreach(list, (GFunc)list_free_wrapper, NULL);
 	g_list_free(list);
 }
 
@@ -988,5 +1013,132 @@ gchar *md5_text_from_file_utf8(const gchar *path, const gchar *error_text)
 	return md5_digest_to_text(digest);
 }
 
+/* Download web file
+ */
+typedef struct _WebData WebData;
+struct _WebData
+{
+	GenericDialog *gd;
+	GCancellable *cancellable;
+	LayoutWindow *lw;
 
+	GtkWidget *progress;
+	GFile *tmp_g_file;
+	GFile *web_file;
+};
+
+static void web_file_async_ready_cb(GObject *source_object, GAsyncResult *res, gpointer data)
+{
+	GError *error = NULL;
+	WebData* web = data;
+	gchar *tmp_filename;
+
+	if (!g_cancellable_is_cancelled(web->cancellable))
+		{
+		generic_dialog_close(web->gd);
+		}
+
+	if (g_file_copy_finish(G_FILE(source_object), res, &error))
+		{
+		tmp_filename = g_file_get_parse_name(web->tmp_g_file);
+		g_free(tmp_filename);
+		layout_set_path(web->lw, g_file_get_path(web->tmp_g_file));
+		}
+	else
+		{
+		file_util_warning_dialog(_("Web file download failed"), error->message, GTK_STOCK_DIALOG_ERROR, NULL);
+		}
+
+	g_object_unref(web->tmp_g_file);
+	web->tmp_g_file = NULL;
+	g_object_unref(web->cancellable);
+	g_object_unref(web->web_file);
+}
+
+static void web_file_progress_cb(goffset current_num_bytes, goffset total_num_bytes, gpointer data)
+{
+	WebData* web = data;
+
+	if (!g_cancellable_is_cancelled(web->cancellable))
+		{
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(web->progress), (gdouble)current_num_bytes / total_num_bytes);
+		}
+}
+
+static void download_web_file_cancel_button_cb(GenericDialog *gd, gpointer data)
+{
+	WebData* web = data;
+
+	g_cancellable_cancel(web->cancellable);
+}
+
+gboolean download_web_file(const gchar *text, gboolean minimized, gpointer data)
+{
+	gchar *scheme;
+	LayoutWindow *lw = data;
+	gchar *tmp_dir;
+	GError *error = NULL;
+	WebData *web;
+	gchar *base;
+	gboolean ret;
+	gchar *message;
+	FileFormatClass format_class;
+
+	scheme = g_uri_parse_scheme(text);
+	if (g_strcmp0("http", scheme) == 0 || g_strcmp0("https", scheme) == 0)
+		{
+		format_class = filter_file_get_class(text);
+
+		if (format_class == FORMAT_CLASS_IMAGE || format_class == FORMAT_CLASS_RAWIMAGE || format_class == FORMAT_CLASS_VIDEO || format_class == FORMAT_CLASS_DOCUMENT)
+			{
+			tmp_dir = g_dir_make_tmp("geeqie_XXXXXX", &error);
+			if (error)
+				{
+				log_printf("Error: could not create temporary file n%s\n", error->message);
+				g_error_free(error);
+				error = NULL;
+				ret = TRUE;
+				}
+			else
+				{
+				web = g_new0(WebData, 1);
+				web->lw = lw;
+
+				web->web_file = g_file_new_for_uri(text);
+
+				base = g_strdup(g_file_get_basename(web->web_file));
+				web->tmp_g_file = g_file_new_for_path(g_build_filename(tmp_dir, base, NULL));
+
+				web->gd = generic_dialog_new(_("Download web file"), "download_web_file", NULL, TRUE, download_web_file_cancel_button_cb, web);
+
+				message = g_strconcat(_("Downloading "), base, NULL);
+				generic_dialog_add_message(web->gd, GTK_STOCK_DIALOG_INFO, message, NULL, FALSE);
+
+				web->progress = gtk_progress_bar_new();
+				gtk_box_pack_start(GTK_BOX(web->gd->vbox), web->progress, FALSE, FALSE, 0);
+				gtk_widget_show(web->progress);
+				if (minimized)
+					{
+					gtk_window_iconify(GTK_WINDOW(web->gd->dialog));
+					}
+
+				gtk_widget_show(web->gd->dialog);
+				web->cancellable = g_cancellable_new();
+				g_file_copy_async(web->web_file, web->tmp_g_file, G_FILE_COPY_OVERWRITE, G_PRIORITY_LOW, web->cancellable, web_file_progress_cb, web, web_file_async_ready_cb, web);
+
+				g_free(base);
+				g_free(message);
+				ret = TRUE;
+				}
+			}
+		}
+	else
+		{
+		ret = FALSE;
+		}
+
+	g_free(scheme);
+	return ret;
+
+}
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
