@@ -1738,7 +1738,6 @@ gchar *text_widget_text_pull_selected(GtkWidget *text_widget)
 		{
 		GtkTextBuffer *buffer;
 		GtkTextIter start, end;
-		GtkTextIter selection_start, selection_end;
 
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_widget));
 		gtk_text_buffer_get_bounds(buffer, &start, &end);
@@ -1759,7 +1758,43 @@ gchar *text_widget_text_pull_selected(GtkWidget *text_widget)
 		{
 		return NULL;
 		}
+}
 
+gboolean defined_mouse_buttons(GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+	LayoutWindow *lw = data;
+	GtkAction *action;
+	gboolean ret = FALSE;
+
+	switch (event->button)
+		{
+		case MOUSE_BUTTON_8:
+			if (options->mouse_button_8)
+				{
+				action = gtk_action_group_get_action(lw->action_group, options->mouse_button_8);
+				if (action)
+					{
+					gtk_action_activate(action);
+					}
+				ret = TRUE;
+				}
+				break;
+		case MOUSE_BUTTON_9:
+			if (options->mouse_button_9)
+				{
+				action = gtk_action_group_get_action(lw->action_group, options->mouse_button_9);
+				if (action)
+					{
+					gtk_action_activate(action);
+					}
+				ret = TRUE;
+				}
+			break;
+		default:
+			break;
+		}
+
+	return ret;
 }
 
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
