@@ -32,10 +32,12 @@
 #include "layout_image.h"
 #include "menu.h"
 #include "metadata.h"
+#include "misc.h"
 #include "thumb.h"
 #include "utilops.h"
 #include "ui_fileops.h"
 #include "ui_menu.h"
+#include "ui_misc.h"
 #include "ui_tree_edit.h"
 #include "uri_utils.h"
 #include "view_file.h"
@@ -666,6 +668,11 @@ gboolean vflist_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer d
 	GtkTreeIter iter;
 	FileData *fd = NULL;
 
+	if (defined_mouse_buttons(widget, bevent, vf->layout))
+		{
+		return TRUE;
+		}
+
 	if (bevent->button == MOUSE_BUTTON_MIDDLE)
 		{
 		vflist_color_set(vf, VFLIST(vf)->click_fd, FALSE);
@@ -789,11 +796,14 @@ static gboolean vflist_select_cb(GtkTreeSelection *selection, GtkTreeModel *stor
 {
 	ViewFile *vf = data;
 	GtkTreeIter iter;
+	GtkTreePath *cursor_path;
 
-	if (!path_currently_selected &&
-	    gtk_tree_model_get_iter(store, &iter, tpath))
+	gtk_tree_view_get_cursor(GTK_TREE_VIEW(vf->listview), &cursor_path, NULL);
+	if (cursor_path)
 		{
+		gtk_tree_model_get_iter(store, &iter, cursor_path);
 		gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &VFLIST(vf)->select_fd, -1);
+		gtk_tree_path_free(cursor_path);
 		}
 	else
 		{
@@ -885,6 +895,7 @@ static void vflist_set_expanded(ViewFile *vf, GtkTreeIter *iter, gboolean expand
 	g_free(sidecars);
 	g_free(name);
 	g_free(formatted);
+	g_free(formatted_with_stars);
 }
 
 static void vflist_setup_iter(ViewFile *vf, GtkTreeStore *store, GtkTreeIter *iter, FileData *fd)
@@ -900,9 +911,9 @@ static void vflist_setup_iter(ViewFile *vf, GtkTreeStore *store, GtkTreeIter *it
 	gboolean expanded = FALSE;
 	gchar *star_rating;
 
-	if (options->show_star_rating)
+	if (options->show_star_rating && fd->rating != STAR_RATING_NOT_READ)
 		{
-		star_rating = metadata_read_rating_stars(fd);
+ 		star_rating = convert_rating_to_stars(fd->rating);
 		}
 	else
 		{
@@ -1254,6 +1265,121 @@ FileData *vflist_thumb_next_fd(ViewFile *vf)
 	return fd;
 }
 
+void vflist_set_star_fd(ViewFile *vf, FileData *fd)
+{
+	GtkTreeStore *store;
+	GtkTreeIter iter;
+	gchar *name;
+	gchar *sidecars;
+	gchar *size;
+	gchar *time;
+	gchar *star_rating;
+	gchar *formatted_with_stars;
+	gboolean expanded;
+
+	if (!fd || vflist_find_row(vf, fd, &iter) < 0) return;
+
+	star_rating = metadata_read_rating_stars(fd);
+
+	store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview)));
+	gtk_tree_store_set(store, &iter, FILE_COLUMN_STAR_RATING, star_rating, -1);
+
+	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+					FILE_COLUMN_NAME, &name,
+					FILE_COLUMN_SIDECARS, &sidecars,
+					FILE_COLUMN_SIZE, &size,
+					FILE_COLUMN_DATE, &time,
+					FILE_COLUMN_EXPANDED, &expanded,
+					-1);
+
+	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, TRUE, star_rating);
+
+	gtk_tree_store_set(store, &iter, FILE_COLUMN_FORMATTED_WITH_STARS, formatted_with_stars,
+					FILE_COLUMN_EXPANDED, expanded,
+					-1);
+
+	g_free(star_rating);
+	g_free(formatted_with_stars);
+}
+
+FileData *vflist_star_next_fd(ViewFile *vf)
+{
+	GtkTreePath *tpath;
+	FileData *fd = NULL;
+	FileData *nfd = NULL;
+
+	/* first check the visible files */
+
+	if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(vf->listview), 0, 0, &tpath, NULL, NULL, NULL))
+		{
+		GtkTreeModel *store;
+		GtkTreeIter iter;
+		gboolean valid = TRUE;
+
+		store = gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview));
+		gtk_tree_model_get_iter(store, &iter, tpath);
+		gtk_tree_path_free(tpath);
+		tpath = NULL;
+
+		while (!fd && valid && tree_view_row_get_visibility(GTK_TREE_VIEW(vf->listview), &iter, FALSE) == 0)
+			{
+			gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &nfd, -1);
+
+			if (nfd && nfd->rating == STAR_RATING_NOT_READ)
+				{
+				fd = nfd;
+				}
+
+			valid = gtk_tree_model_iter_next(store, &iter);
+			}
+
+		if (fd)
+			{
+			vf->stars_filedata = fd;
+
+			if (vf->stars_id == 0)
+				{
+				vf->stars_id = g_idle_add_full(G_PRIORITY_LOW, vf_stars_cb, vf, NULL);
+				}
+			}
+		}
+
+	/* then find first undone */
+
+	if (!fd)
+		{
+		GList *work = vf->list;
+
+		while (work && !fd)
+			{
+			FileData *fd_p = work->data;
+
+			if (fd_p && fd_p->rating == STAR_RATING_NOT_READ)
+				{
+				fd = fd_p;
+				}
+			else
+				{
+				fd = NULL;
+				}
+
+			work = work->next;
+			}
+
+		if (fd)
+			{
+			vf->stars_filedata = fd;
+
+			if (vf->stars_id == 0)
+				{
+				vf->stars_id = g_idle_add_full(G_PRIORITY_LOW, vf_stars_cb, vf, NULL);
+				}
+			}
+		}
+
+	return fd;
+}
+
 /*
  *-----------------------------------------------------------------------------
  * row stuff
@@ -1318,7 +1444,7 @@ static gboolean vflist_row_is_selected(ViewFile *vf, FileData *fd)
 		if (fd_n == fd) found = TRUE;
 		work = work->next;
 		}
-	g_list_foreach(slist, (GFunc)gtk_tree_path_free, NULL);
+	g_list_foreach(slist, (GFunc)tree_path_free_wrapper, NULL);
 	g_list_free(slist);
 
 	return found;
@@ -1365,7 +1491,7 @@ guint vflist_selection_count(ViewFile *vf, gint64 *bytes)
 		}
 
 	count = g_list_length(slist);
-	g_list_foreach(slist, (GFunc)gtk_tree_path_free, NULL);
+	g_list_foreach(slist, (GFunc)tree_path_free_wrapper, NULL);
 	g_list_free(slist);
 
 	return count;
@@ -1407,7 +1533,7 @@ GList *vflist_selection_get_list(ViewFile *vf)
 
 		work = work->next;
 		}
-	g_list_foreach(slist, (GFunc)gtk_tree_path_free, NULL);
+	g_list_foreach(slist, (GFunc)tree_path_free_wrapper, NULL);
 	g_list_free(slist);
 
 	return g_list_reverse(list);
@@ -1437,7 +1563,7 @@ GList *vflist_selection_get_list_by_index(ViewFile *vf)
 
 		work = work->next;
 		}
-	g_list_foreach(slist, (GFunc)gtk_tree_path_free, NULL);
+	g_list_foreach(slist, (GFunc)tree_path_free_wrapper, NULL);
 	g_list_free(slist);
 
 	return g_list_reverse(list);
@@ -1667,7 +1793,7 @@ void vflist_selection_to_mark(ViewFile *vf, gint mark, SelectionToMarkMode mode)
 
 		work = work->next;
 		}
-	g_list_foreach(slist, (GFunc)gtk_tree_path_free, NULL);
+	g_list_foreach(slist, (GFunc)tree_path_free_wrapper, NULL);
 	g_list_free(slist);
 }
 
@@ -1746,6 +1872,7 @@ static void vflist_populate_view(ViewFile *vf, gboolean force)
 	store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview)));
 
 	vf_thumb_stop(vf);
+	vf_star_stop(vf);
 
 	if (!vf->list)
 		{
@@ -1770,6 +1897,7 @@ static void vflist_populate_view(ViewFile *vf, gboolean force)
 
 	vf_send_update(vf);
 	vf_thumb_update(vf);
+	vf_star_update(vf);
 }
 
 gboolean vflist_refresh(ViewFile *vf)
@@ -1803,6 +1931,10 @@ gboolean vflist_refresh(ViewFile *vf)
 		vf->list = file_data_filter_marks_list(vf->list, vf_marks_get_filter(vf));
 		vf->list = g_list_first(vf->list);
 		vf->list = file_data_filter_file_filter_list(vf->list, vf_file_filter_get_filter(vf));
+
+		vf->list = g_list_first(vf->list);
+		vf->list = file_data_filter_class_list(vf->list, vf_class_get_filter(vf));
+
 		file_data_register_notify_func(vf_notify_cb, vf, NOTIFY_PRIORITY_MEDIUM);
 
 		DEBUG_1("%s vflist_refresh: sort", get_exec_time());
@@ -2005,6 +2137,7 @@ void vflist_destroy_cb(GtkWidget *widget, gpointer data)
 	vflist_select_idle_cancel(vf);
 	vf_refresh_idle_cancel(vf);
 	vf_thumb_stop(vf);
+	vf_star_stop(vf);
 
 	filelist_free(vf->list);
 }
@@ -2052,7 +2185,7 @@ ViewFile *vflist_new(ViewFile *vf, FileData *dir_fd)
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(vf->listview), FALSE);
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(vf->listview), FALSE);
 
-	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(vf->listview), FILE_COLUMN_FORMATTED);
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(vf->listview), -1);
 
 	column = 0;
 
